/*****************************************************************/
/*      communicator.h                                           */
/*	Henri Casanova					         */
/*---------------------------------------------------------------*/
/*****************************************************************/

#ifndef __NS_COMMUNICATOR_H
#define __NS_COMMUNICATOR_H

#include "general.h"

#ifdef MULTINODESEQ
# include "ibp_base.h"
#endif

/**
 **  MACRO DEFINITIONS
 **/

/* Buffer size for the file accesses */
#define BUFFSIZE 8192

/* types of transfers: XDR or not */
#define DATA_RAW 0
#define DATA_XDR 1

/* data types */
#define NETSOLVE_NOTYPE   0 /* No type        */
#define NETSOLVE_I        1 /* Integer        */
#define NETSOLVE_S        2 /* Simple         */
#define NETSOLVE_D        3 /* Double         */
#define NETSOLVE_C        4 /* Simple complex */
#define NETSOLVE_Z        5 /* Double complex */
#define NETSOLVE_CHAR     6 /* Unsigned char  */
#define NETSOLVE_B        7 /* Byte (no XDR)  */


/**
 **  STRUCTURE DEFINITIONS
 **/

typedef struct NS_Timer_ {
  struct timeval start;
  int running;
  double accum;
} NS_Timer;

void init_timing(NS_Timer*);
void start_timing(NS_Timer*);
void stop_timing(NS_Timer*);
void print_timing(const NS_Timer*);

typedef struct NS_Communicator_ NS_Communicator;

/*  Communicator 
 *
 *  This structure contains the socket descriptor
 *  and the current encoding
 */

struct NS_Communicator_ {
#ifndef WIN32
  int sock;
#else /* WIN32 */
  SOCKET sock;
#endif /* WIN32 */

  int encoding;
};

/*  scomplex and dcomplex
 *
 * Faking complex data types
 */
typedef struct {
  float r;
  float i;
} scomplex;

typedef struct {
  double r;
  double i;
} dcomplex;


/**
 **  FUNCTION PROTOTYPES
 **/

int setXDRSizes();

#ifndef WIN32
NS_Communicator *newCommunicator(int,int);
NS_Communicator *initTransaction(int,int);
NS_Communicator *acceptTransaction(int);
#else /* WIN32 */
NS_Communicator *newCommunicator(SOCKET,int);
NS_Communicator *initTransaction(SOCKET,int);
NS_Communicator *acceptTransaction(SOCKET);
#endif /* WIN32 */
void endTransaction(NS_Communicator*);
int sendArray(NS_Communicator*,int,void*,int);
int recvArray(NS_Communicator*,int,void*,int);
int sendInt(NS_Communicator*,int);
int recvInt(NS_Communicator*,int*);
#ifdef MULTINODESEQ
int sendSetOfCaps(NS_Communicator*,struct ibp_set_of_caps*);
int recvSetOfCaps(NS_Communicator*,struct ibp_set_of_caps**);
int sendArrayFromIBP(NS_Communicator*,int,struct ibp_depot*, struct ibp_set_of_caps*,ulong_t*,int);
int recvArrayToIBP(NS_Communicator*,int,struct ibp_depot*,struct ibp_set_of_caps*,int);
int sendIntIBP(struct ibp_depot*, struct ibp_set_of_caps*,int);
int recvIntIBP(struct ibp_depot*, struct ibp_set_of_caps*,ulong_t*,int*);
int sendSetOfCapsIBP(struct ibp_depot*, struct ibp_set_of_caps*, struct ibp_set_of_caps*);
int recvSetOfCapsIBP(struct ibp_depot*, struct ibp_set_of_caps*, ulong_t*, struct ibp_set_of_caps**);
int sendStringIBP(struct ibp_depot * depot, struct ibp_set_of_caps * caps, char *s);
int recvStringIBP(struct ibp_depot * depot, struct ibp_set_of_caps * caps, ulong_t*, char **s);
int sendArrayIBP(struct ibp_depot * depot, struct ibp_set_of_caps * caps,
		 int data_type, void *data,int nb);
int recvArrayIBP(struct ibp_depot*,struct ibp_set_of_caps*,ulong_t*,int,void*,int);
#else
int sendArrayFromFile(NS_Communicator*,int,int,int);
int recvArrayToFile(NS_Communicator*,int,int,int);
#endif
int sendString(NS_Communicator*,char*);
int recvString(NS_Communicator*,char**);
int sendIPaddr(NS_Communicator*,NS_IPaddr_type*);
int recvIPaddr(NS_Communicator*,NS_IPaddr_type*);
int netsolve_sizeof(int);
int netsolve_xdrsizeof(int);
int sendFileAsString(NS_Communicator*,char*);

int recv8BitFlag(NS_Socket_type,int*);
int send8BitFlag(NS_Socket_type,int);

#endif

