/* $Id: nws_nameserver.h,v 1.2 2000/03/28 15:03:45 susan Exp $ */

#ifndef NAMESERVER_H
#define NAMESERVER_H


#include "messages.h"  /* Message number ranges */


/*
** Messages recognized and/or generated by nws_nameserver.
**
** NS_FAILED(void) -- sent from the nws_nameserver process as the unsuccessful
**   response to either a NS_SEARCH or a NS_REGISTER message.
** NS_REGISTER(object,timeOut) -- sent to the nws_nameserver process to request
**   that it register #object#, a tab-delimited, double-tab-terminated list of
**   attribute:value pairs, for the next #timeOut# (unsigned long) seconds.  A
**   #timeOut# of zero indicates that the registration should never expire.
**   nws_nameserver responds with either NS_REGISTERED or NS_FAILED depending
**   on success.
** NS_REGISTERED(void) -- sent from the nws_nameserver process as the
**   successful reply to a NS_REGISTER message.
** NS_SEARCH(filter) -- sent to the nws_nameserver process to request that it
**   search registered objects for those that match #filter#, a nul-terminated,
**   LDAP-style search filter.  nws_nameserver replies with either NS_SEARCHED
**   or NS_FAILED depending on success.
** NS_SEARCHED(objectList) -- sent from the nws_nameserver process as the
**   successful reply to a NS_SEARCH message.  #objectList# is a tab-delmited
**   list of attribute:value pairs where each object is terminated by a pair of
**   tabs (i.e. an empty attribute:value pair).  This list contains all
**   registered objects that matched the search filter.
** NS_UNREGISTER(filter) -- sent to the nws_nameserver process to request that
**   it delete all registrations for items that match #filter#, a nul-
**   terminated, LDAP-style search filter.  nws_nameserver replies with a
**   NS_UNREGISTERED message.
** UN_UNREGISTERED(void) -- sent from the nws_nameserver process as the
**   successful reply to a NS_UNREGISTER message.
*/

#define NS_REGISTER     NAMESERVER_FIRST_MESSAGE
#define NS_REGISTERED   (NS_REGISTER + 1)
#define NS_SEARCH       (NS_REGISTERED + 1)
#define NS_SEARCHED     (NS_SEARCH + 1)
#define NS_UNREGISTER   (NS_SEARCHED + 1)
#define NS_UNREGISTERED (NS_UNREGISTER + 1)

#define NS_FAILED     NAMESERVER_LAST_MESSAGE


#endif
