/*****************************************************************/
/*      agent.c                                                  */
/*      Henri Casanova, Dorian Arnold                            */
/*---------------------------------------------------------------*/
/*  main()                                                       */
/*****************************************************************/

#include "core.h"
#include <signal.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/socket.h>
#include "agentglobal.h"
#include "agentinit.h"
#include "agentprocessmessage.h"
#include <time.h>
#include "infoserver.h"

/*
 *  This is the main routine. It does nothing but 
 *  calling the initialization routine and
 *  waiting for a network connection.
 */

int main(int argc,char **argv)
{
  time_t cur_time, last_check;
  NS_Socket_type sock;

  /* Initialize the agent */
  if (agent_init(argc,argv) == -1)
    exit(0);

#ifdef INFOSERVER
  /*
   * If we want to use the infoserver, AND we want
   * to run it locally, then fork it off, otherwise,
   * either remember where it is, or don't do a thing.
   */
#ifdef STANDALONEISERV
  fprintf(stderr, "Will use standalone InfoServer\n");
  remote_infoserver("localhost", 0);
#else
  fprintf(stderr, "Will spawn the InfoServer\n");
  spawn_infoserver();
#endif
#endif

  /* at least 2 min. delay before 1st check */
  last_check = time(0) + 60;

  listen(global.sock,MAX_CONNECTIONS);
  while(1)
  {
    if ((sock = acceptConnection(global.sock)) == -1)
      continue;

    cur_time = time(0);
    if(cur_time - last_check >= 60){
      printf("checking servers: %s\n", Ctime(&cur_time));
      check_servers(cur_time);
      last_check = time(0);
      /*
       * Just call jobid_tidy() at regular intervals...
       * Once a day would be fine actually...
       */
      jobid_tidy();
    }
    fflush(stderr);
    fflush(stdout);

    processMessage(sock);
    fflush(stderr);
    fflush(stdout);
  }
}
