/*****************************************************************/
/*      calling_sequence.c                                       */
/*      Version 1.0                      Henri Casanova          */
/*****************************************************************/

#include <stdarg.h>
#include "core.h"
#include "mnemonics.h"
#include "expressions.h"
#include "callingsequence.h"
#include "cfortranclient.h"

/*
 * assignConstantValues()
 */

int assignConstantValues(NS_ProblemDesc *pd,NS_Object **input,NS_Object **output)
{
  int value;
  int i;

  for (i=0;i<pd->calling_sequence->nb_constants;i++)
  {
    value = atoi(pd->calling_sequence->constants[i].value);
    assignValue(pd,input,output,pd->calling_sequence->constants[i].mnemonic,&value); 
  }
  return 1;
}

/*
 * assignFormulaValues()
 */
int assignFormulaValues(NS_ProblemDesc *pd,NS_Object **input,NS_Object **output)
{
  int value;
  int i;

  for (i=0;i<pd->calling_sequence->nb_formulae;i++)
  {
    value = evalFormula(pd,pd->calling_sequence->formulae[i].expression);
    assignValue(pd,input,output,pd->calling_sequence->formulae[i].mnemonic,&value);
  }
  return 1;
}

/*
 * parseStack()
 */
int parseStack(int language,NS_ProblemDesc *pd,va_list argptr)
{
  NS_VoidStarOrInt *array;
  
  if (stackToArray(language,pd,argptr,&array) == -1)
    return -1;
  if (arrayToObjects(language,pd,pd->input_objects,pd->output_objects,array) == -1)
  {
    free(array);
    return -1;
  }
  free(array);
  return 1;
}

/*
 * stackToArray()
 * after the problem nickname and the possible FORTRAN return value stuff
 * one can parse the stack and put things in an array.
 */
int stackToArray(int language,NS_ProblemDesc *pd,va_list argptr,NS_VoidStarOrInt **array)
{
  int i;
  char *mnemoniclist;
  NS_VoidStarOrInt *new;
  int arg_type;
  char *char_ptr[256];
  int string_len[256];
  int arg_index_for_char_ptr[256];
  int char_ptr_index = 0;
  int shadow;
  int isstring;
  
 
  new = (NS_VoidStarOrInt *)calloc(pd->calling_sequence->nb_args,sizeof(NS_VoidStarOrInt));

  for (i=0;i<pd->calling_sequence->nb_args;i++)
  {
    mnemoniclist = strdup(pd->calling_sequence->arg_to_mnemonic[i]);
    arg_type = getArgumentType(mnemoniclist);
    /* WARNING : If a String is ignored It DOES NOT WORK !!! */
    if (arg_type == IGNORE)
    {
      (void)va_arg(argptr,void*);
      new[i].ptr = NULL;
      free(mnemoniclist);
      continue;
    }
    if (arg_type == INTEGER)
    {
      if (language == CALL_FROM_FORTRAN)
      {
        int *tmp;
        tmp = (int*)va_arg(argptr,int*);
        new[i].i = *tmp;
      } else {
        new[i].i = (int)va_arg(argptr,int);
      }
    }
    if (arg_type == POINTER)
    {
      new[i].ptr = (void*)va_arg(argptr,void*);
    }
  
    /* From C: No need to take care of the strlen shit */
    if (language == CALL_FROM_C)
    {
      free(mnemoniclist);
      continue;
    }

    /* If the current arg is not a string, we are FINE */
    isstring = isStringArgument(pd,mnemoniclist);
    if (isstring == 0)
    {
      free(mnemoniclist);
      continue;
    }
#ifdef PLAT_T3E
    if (isstring == 2) /* Only on CRAY */
    {
      /* POP The USELESS LENGTH */
      int dummy=((int)va_arg(argptr,int));
      continue;
    }
#endif

    /* Ok, it's a string..... sigh .... */
    char_ptr[char_ptr_index] = (char*)new[i].ptr;
    arg_index_for_char_ptr[char_ptr_index] = i;
#ifdef STACK_SCHEME_CRAY
    string_len[char_ptr_index] = ((int)va_arg(argptr,int));
#endif 
    char_ptr_index++;
    free(mnemoniclist);
  }  
  if (language == CALL_FROM_FORTRAN)
  {
    char *new_string;
#ifdef STACK_SCHEME_SUN
    /* Get all the strlen */

    /* Pop the length of the problem nickname */
    shadow = (int)va_arg(argptr,int);
    for (i=0;i<char_ptr_index;i++)
    {
      string_len[i] = (int)va_arg(argptr,int);
    }
#endif

    /* replace the regular string with NULL terminated ones */
    for (i=0;i<char_ptr_index;i++)
    {
      new_string = (char*)calloc(string_len[i]+1,sizeof(char));
      strncpy(new_string,char_ptr[i],string_len[i]); 
      new[arg_index_for_char_ptr[i]].ptr = new_string;
    }
  }

  *array = new;
  return pd->calling_sequence->nb_args;
}

/*
 * arrayToObjects()
 *
 * Use the array generated by the function above to 
 * fill in the objects
 */
int arrayToObjects(int language, NS_ProblemDesc *pd,
            NS_Object **input,NS_Object **output,NS_VoidStarOrInt *array)
{
  int i;
  char *mnemoniclist;
  int arg_type;
  void *arg_value = 0;
  char *mnemonic;

  /* Putting the values in the objects */
  for (i=0;i<pd->calling_sequence->nb_args;i++)
  {
    mnemoniclist = strdup(pd->calling_sequence->arg_to_mnemonic[i]);
    arg_type = getArgumentType(mnemoniclist);

    if (arg_type == IGNORE)
    {
      free(mnemoniclist);
      continue;
    }
    if (arg_type == INTEGER)
      arg_value = (void*)(&(array[i].i));
    if (arg_type == POINTER)
      arg_value = array[i].ptr;

    mnemonic = strtok(mnemoniclist," \t,\0");
    while(mnemonic != NULL)
    {
      if (language == CALL_FROM_FORTRAN)
      {
        int index;
        index = getIndex(mnemonic);
        if (getIO(mnemonic) == OUTPUT)
        {
          if (output[index]->object_type == NETSOLVE_STRING)
            output[index]->attributes.string_attributes.ptr = arg_value;
          else
            assignValue(pd,input,output,mnemonic,arg_value);  
        }
        else 
        {
          if (input[index]->object_type == NETSOLVE_STRING)
            assignValue(pd,input,output,mnemonic,strdup((char*)arg_value));
          else
            assignValue(pd,input,output,mnemonic,arg_value);
        }
      }
      else
        assignValue(pd,input,output,mnemonic,arg_value);
      mnemonic = (char *)strtok(NULL,"\t ,");
    }
    free(mnemoniclist); 
  }
  return 1;
}
