/*****************************************************************/
/*      fnetsl.c                                                 */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  fnetsl()                                                     */
/*****************************************************************/

/*
 * fnetsl() : vararg C Interface to the NetSolve Client
 */

#include <stdarg.h>
#include "netsolve.h"
#include "core.h"
#include "client.h"
#include "callingsequence.h"
#include "cfortranclient.h"
#include "mnemonics.h"

#ifdef ADD_
void fnetsl_(char *nickname,...)
#endif
#ifdef NOCHANGE
void fnetsl(char *nickname,...)
#endif
#ifdef UPCASE
void FNETSL(char *nickname,...)
#endif
{
  int len;
  NS_ProblemDesc *pd;
  int status;
  va_list argptr;
  char *buf;
  int *return_value;
  NS_VoidStarOrInt *call;

  if(!proxy_pid){  /*proxy needs to be started*/
    if(netslinit(NULL) < 0){
      /* mm - wrong return type return ns_errno; */
      return;
    }
  }

  /* Getting the problem's nickname */
  va_start(argptr,nickname); 

#ifdef STACK_SCHEME_CRAY
    va_arg(argptr,int);
#endif

  /* Get the address of the return value */
  return_value = (int*)va_arg(argptr,int*);

  /* Stripping off the parenthesis */


  buf = nickname;
  while ((*buf != '(')&&(*buf != '\0'))
    buf++;

  len = (int)(buf - nickname+2);
  if (nickname[len-2] != '(')
  {
    ns_errno = NetSolveBadProblemName;
    *return_value = NetSolveBadProblemName;
    return;
  }
  if (nickname[len-1] != ')')
  {
    ns_errno = NetSolveBadProblemName;
    *return_value = NetSolveBadProblemName;
    return;
  }

  buf = (char *)calloc(len-1,sizeof(char));
  BCOPY(nickname,buf,len-2);

  /* Initializing the major    */
  setMajorDefault("Col");

  /* Getting the problem descriptor */
  status = netsolveInfo(buf,&pd);
  free(buf);
  if (status == -1)
  {
    *return_value = ns_errno;
    return;
  }

    /* Parsing the stack */
  if (stackToArray(CALL_FROM_FORTRAN,pd,argptr,&call) == -1)
  {
    freeProblemDesc(pd);
    *return_value = ns_errno;
    return;
  }
  netslX(pd,call);
  free(call);
  freeProblemDesc(pd);
  *return_value = ns_errno;
  return;
}
