/*****************************************************************/
/*      netsl.c                                                  */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netsl()                                                      */
/*****************************************************************/

/*
 * netsl() : vararg C Interface to the NetSolve Client
 */

#include <stdarg.h>
#include "core.h"
#include "client.h"
#include "mnemonics.h"
#include "callingsequence.h"
#include "cfortranclient.h"
#include "sequences.h"
#include "netsolve.h"
#include "client2proxy.h"

int netsl(char *nickname,...)
{
  int len;
  NS_ProblemDesc *pd;
  int status;
  va_list argptr;
  char *buf;
  NS_VoidStarOrInt *call;

  if(!proxy_pid){  /*proxy needs to be started*/
    if(netslinit(NULL) < 0){
      return ns_errno;
    }
  }

  /* Getting the problem's nickname */
  va_start(argptr,nickname); 
  buf = strdup(nickname);

  /* Stripping off the parenthesis */
  len = strlen(buf);
  if (buf[len-2] != '(')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  if (buf[len-1] != ')')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  buf[len-2] = '\0';

  /* Initializing the major */
  setMajorDefault("Row");

#ifdef WIN32
  /* Initialize WinSock */
  if ((!WinSockInit()))
  {
    ns_errno = NetSolveNetworkError;
    return ns_errno;
  }
#endif /* WIN32 */


#ifdef DEBUG
  fprintf (stderr, "Getting problem descriptor... call netsolveInfo\n");
#endif

  /* Getting the problem descriptor */
  status = netsolveInfo(buf,&pd);
  free(buf);
  if (status == -1)
    return ns_errno;

#ifdef DEBUG
  fprintf (stderr, "Parsing the stack... call stackToArray\n");
#endif

  /* Allocate globally unique job_id */
  /* XXX: QUERY PROXY FOR UNIQUE ID - HAVE IT ASK THE AGENT */
  /* Remember to put this stuff into fnetsl.c as well (sheesh...) */
  pd->job_id = CP_allocJobId();

  /* Parsing the stack */
  if (stackToArray(CALL_FROM_C,pd,argptr,&call) == -1)
  { 
    freeProblemDesc(pd);
    return ns_errno;
  } 
#ifdef DEBUG
  fprintf (stderr, "call netslX\n");
#endif

  netslX(pd,call);
  free(call);
  if (!(netsl_sequence_status))  freeProblemDesc(pd); /* don't free if sequencing */
  return ns_errno;
}
