/*****************************************************************/
/*      netslnb.c                                                */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netslnb()                                                    */
/*****************************************************************/

/*
 * netslnb() : vararg C Interface to the NetSolve Client
 */

#include <stdarg.h>
#include "netsolve.h"
#include "core.h"
#include "client.h"
#include "callingsequence.h"
#include "cfortranclient.h"
#include "mnemonics.h"

int netslnb(char *nickname,...)
{
  int len;
  NS_ProblemDesc *pd;
  int status;
  va_list argptr;
  char *buf;
  NS_VoidStarOrInt *call;

  if(!proxy_pid){  /*proxy needs to be started*/
    if(netslinit(NULL) < 0){
      return ns_errno;
    }
  }

  /* Getting the problem's nickname */
  va_start(argptr,nickname); 
  buf = strdup(nickname);

  /* Stripping off the parenthesis */
  len = strlen(buf);
  if (buf[len-2] != '(')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  if (buf[len-1] != ')')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  buf[len-2] = '\0';

#ifdef WIN32
  /* Initialize WinSock */
  if ((!WinSockInit()))
  {
    ns_errno = NetSolveNetworkError;
    return ns_errno;
  }
#endif /* WIN32 */

  /* Getting the problem descriptor */
  status = netsolveInfo(buf,&pd);
  free(buf);
  if (status == -1)
    return ns_errno;


  /* Using the calling sequence description */
  if (stackToArray(CALL_FROM_C,pd,argptr,&call) == -1)
  { 
    freeProblemDesc(pd);
    return ns_errno;
  } 
  return netslnbX(pd,call);
}
