/*****************************************************************/
/*      netslnb_assignment.c                                     */
/*      Version 1.0                      Henri Casanova          */
/*---------------------------------------------------------------*/
/*  netslnb_assignment()                                         */
/*****************************************************************/

/*
 * netslnb_assignment() : vararg C Interface to the NetSolve Client
 *                        for request assignments
 */

#include <stdarg.h>
#include "core.h"
#include "client.h"
#include "callingsequence.h"
#include "cfortranclient.h"
#include "mnemonics.h"

int netslnb_assignment(char *string,...)
{
  int len;
  NS_ProblemDesc *pd;
  int status;
  va_list argptr;
  char *buf;
  NS_VoidStarOrInt *call;
  char *nickname, *IPaddr;

  /* Getting the problem's string */
  va_start(argptr,string); 
  buf = strdup(string);

  /* Stripping off the parenthesis */
  len = strlen(buf);
  if (buf[len-2] != '(')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  if (buf[len-1] != ')')
  {
    free(buf);
    return NetSolveBadProblemName;
  }
  buf[len-2] = '\0';

#ifdef WIN32
  /* Initialize WinSock */
  if ((!WinSockInit()))
  {
    ns_errno = NetSolveNetworkError;
    return ns_errno;
  }
#endif /* WIN32 */

  /* Extracting the nickname, IPaddr */
  IPaddr = strtok(buf,":");
  nickname = strtok(NULL,"\0");
  
  if ((IPaddr == NULL) || 
      (nickname == NULL))
  {
    free(buf);
    return NetSolveBadProblemName;
  }

  /* Getting the problem descriptor */
  status = netsolveInfo(nickname,&pd);
  if (status == -1)
  {
    free(buf);
    return ns_errno;
  }


  /* Using the calling sequence description */
  if (stackToArray(CALL_FROM_C,pd,argptr,&call) == -1)
  { 
    free(buf);
    freeProblemDesc(pd);
    return ns_errno;
  } 
  status = netslnbX_assignment(getIPaddr(IPaddr),pd,call);
  free(buf);
  return status;
}
