/*****************************************************************/
/*      CP_allocjobid.c                                          */
/*      Jakob Oestergaard                                        */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "client.h"
#include "client2proxy.h"
#include "errno.h"
#include "netsolve.h"
#include "proxyjobdesc.h"
#include "proxy.h"
#include <signal.h>
#include <time.h>

/*
 *  CP_allocJobId()
 *
 * Asks the proxy to ask the agent for job_id.
 * returns 0 on failure (because that's an
 * invalid job_id)
 */
int CP_allocJobId(NS_RequestDesc *rd)
{
  NS_Communicator *comm;
  int job_id;

  /* contact the proxy */
  comm = contactProxy(1);
  if(!comm) {
    netsolvePerror(__FILE__,__LINE__);
    ns_errno = NetSolveProxyError;
    return 0;
  } 

  /* send request */
  if(-1 == sendInt(comm, NS_PROT_ALLOC_JOBID)) {
    netsolvePerror(__FILE__,__LINE__);
    ns_errno = NetSolveProxyError;
    return 0;
  }

  /* await reply */
  if(-1 == recvInt(comm, &ns_errno)) {
    fprintf(stderr, "Reception of status for allocJobId() failed\n");
    ns_errno = NetSolveNetworkError;
    return 0;
  }
  if(ns_errno != NetSolveOK) {
    return 0;
  }
  if(-1 == recvInt(comm, &job_id)) {
    ns_errno = NetSolveNetworkError;
    return 0;
  }
  return job_id;
}
