/*****************************************************************/
/*      CP_awaitjobcompletion.c                                  */
/*      Jakob Oestergaard                                        */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "client.h"
#include "client2proxy.h"
#include "errno.h"
#include "netsolve.h"
#include "proxyjobdesc.h"
#include "proxy.h"
#include <signal.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <sys/stat.h>

/*
 *  CP_awaitJobCompletion()
 *
 *  Blocks until the given rd either
 *  completes with success or fails
 */
int CP_awaitJobCompletion(NS_RequestDesc *rd)
{
  int error_code;
  NS_Communicator *comm;
  int NS_errno;
  int magic;
  int rc;

  /********************************************/
  /* contact the proxy 			      */
  /********************************************/
  comm = contactProxy(1);
  if (comm == NULL)
  {
    netsolvePerror(__FILE__,__LINE__);
    ns_errno = NetSolveProxyError;
    return -1;
  } 

  /*
   * Sending
   *
   * We ask the proxy to let us know when the request_id
   * completes.
   * In case of sequencing, the same request ID will result
   * in multiple PartialCompletionOK events
   */
  if ((sendInt(comm,NS_PROT_AWAIT_COMPLETION) == -1)  ||
      (sendInt(comm,rd->request_id) == -1))
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  }

  /* Receiving */
  if (recvInt(comm,&NS_errno) == -1)
  {
    fprintf(stderr, "recvInt returned -1: NS_errno = %i\n", NS_errno);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  } 
  if (NS_errno != NetSolveOK)
  {
    fprintf(stderr, "recvInt returned ok: NS_errno = %i\n", NS_errno);
    endTransaction(comm);
    ns_errno = NS_errno;
    return -1;
  }

  magic = 0xBADBAD;
  if((rc = recvInt(comm,&magic)) == -1) {
    fprintf(stderr, "recvInt returned -1: NS_errno = %i\n", NS_errno);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  } 
  /* ACK it */
  if(sendInt(comm,magic) == -1) {
    fprintf(stderr, "sendInt returned -1: NS_errno = %i\n", NS_errno);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  } 

  if(magic != WAITMAGIC) {
    fprintf(stderr, "%i, CAUGHT BAD MAGIC (0x%X = %i) rc = %i !!\n", 
	    getpid(), magic, magic, rc);
    ns_errno = NetSolveProxyError;
    endTransaction(comm);
    return -1;
  }

  if (recvInt(comm,&error_code) == -1) /* The actual job status */
  {
    fprintf(stderr, "recvInt returned -1, error_code = %i\n", error_code);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  }

  if (recvInt(comm,&rd->pd->job_id) == -1) /* The job_id */
  {
    fprintf(stderr, "recvInt returned -1, error_code = %i\n", error_code);
    endTransaction(comm);
    ns_errno = NetSolveProxyError;
    return -1;
  }
  endTransaction(comm);

  ns_errno = error_code;
  if (ns_errno == NetSolveOK)
    return 1;
  if (ns_errno == NetSolvePartialCompletionOK)
    return 0;
  return -1;
}
