/*****************************************************************/
/*      waitassignment.c                                         */
/*      Henri Casanova                                           */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "client.h"
#include "client2proxy.h"
#include "client2server.h"
#include <unistd.h>
#include <signal.h>

/*
 * netsolveWaitAssignment()
 * 
 * This functions waits for an assignment completion, and does not
 * resubmit the job in case of a failure.
 */
int netsolveWaitAssignment(NS_RequestDesc **rd,int *elapsed)
{
  int status;

  /* Polling the proxy !!! Not really cool, but works */
  while(1) {
    status = CP_probeJob(*rd);
    if (ns_errno != NetSolveNotReady)
      break;
    sleep(3); 
  }

  if (status == -1)
  {
    reportFailure((*rd)->IPaddr,(*rd)->pd->nickname,
                  (*rd)->request_id,SERVER_ERROR,(*rd)->pd->job_id);
    return -1; 
  }
  
  if (CS_downloadOutput(*rd,elapsed) == -1)
  { 
    switch (ns_errno) 
    {
      case NetSolveBadValues:
        return -1;
      case NetSolveDimensionMismatch:
        return -1;
      case NetSolveNoSolution:
        return -1;
    }
    return -1;
  }

  CP_reportJobCompleted((*rd)->IPaddr,(*rd)->request_id,(*rd)->pd->job_id);

  ns_errno = NetSolveOK;
  return 1; 
}

