/*****************************************************************/
/*      waitrequest.c                                            */
/*      Henri Casanova                                           */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "client.h"
#include "client2proxy.h"
#include "client2server.h"
#include <unistd.h>
#include <signal.h>

/*
 * netsolveWaitRequest()
 * 
 * This function waits for the completion of a request and automatically
 * resubmit the job in case of an error.
 */
int netsolveWaitRequest(NS_RequestDesc **rd,int *elapsed)
{
  int status;
  NS_RequestDesc *newrd;

  /*
   * Await completion
   */
  status = CP_awaitJobCompletion(*rd);

  /* 
   * Let the proxy know that it can expire this ID
   */
  CP_retireJob(*rd);

  if (status == -1)
  {
    /* Resubmit */
    if (ns_errno == NetSolveInvalidRequestID)
    {
      *rd = NULL;
      *elapsed = 0;
      return -1;
    }
    reportFailure((*rd)->IPaddr, (*rd)->pd->nickname,
                  (*rd)->request_id,SERVER_ERROR,(*rd)->pd->job_id);
    newrd = netsolveSendRequest((*rd)->pd,(*rd)->input,(*rd)->output);
    free(*rd);
    *rd = newrd;
    if (*rd == NULL)
      return -1;
    return netsolveWaitRequest(rd,elapsed);
  }
  
  if (CS_downloadOutput(*rd,elapsed) == -1)
  { 
    switch (ns_errno) 
    {
      case NetSolveBadValues:
        return -1;
      case NetSolveDimensionMismatch:
        return -1;
      case NetSolveNoSolution:
        return -1;
    }
    /* Resubmit */
    reportFailure((*rd)->IPaddr,(*rd)->pd->nickname,
                  (*rd)->request_id,HOST_ERROR, (*rd)->pd->job_id);
    newrd = netsolveSendRequest((*rd)->pd,(*rd)->input,(*rd)->output);
    free(*rd);
    *rd = newrd;
    if (*rd == NULL)
      return -1;
    return netsolveWaitRequest(rd,elapsed);
  }

  CP_reportJobCompleted((*rd)->IPaddr,(*rd)->request_id,(*rd)->pd->job_id);

  ns_errno = NetSolveOK;
  return 1; 
}

