/*****************************************************************/
/*      netsolveerror.c						 */
/*      Henri Casanova						 */
/*****************************************************************/

#include "core.h"

/*
 * netsolvePerror()
 */
void netsolvePerror(char *file, int line)
{
  fprintf(STDERR "%s,%d: %s\n",file,line,netsolveErrorMessage(ns_errno));
  return;
}

/*
 * netsolveErrorMessage()
 */
char *netsolveErrorMessage(int e)
{
  switch(e)
  {
    case NetSolveBadIterationRange:
      return ("NetSolve Farming: Invalid iteration range");
      break;
    case NetSolveCannotContactAgent:
      return ("NetSolve : Cannot contact agent");
      break;
    case NetSolveProxyError:
      return ("NetSolve : Error while talking to proxy");
      break;
    case NetSolveUnknownServer:
      return ("NetSolve : Unknown server");
      break;
    case NetSolveCannotStartProxy:
      return ("NetSolve : Cannot start proxy");
      break;
    case NetSolveFarmingError:
      return ("NetSolve Farming: One or more requests failed");
      break;
    case NetSolveOK:
      return ("NetSolve: no error");
      break;
    case NetSolveCondorError:
      return ("NetSolve: Condor error");
      break;
    case NetSolveConnectionRefused:
      return ("NetSolve: connection refused");
      break;
    case NetSolveUPFUnsafe:
      return ("NetSolve: UPF security violation");
      break;
    case NetSolveServerError:
      return ("NetSolve: server error");
      break;
    case NetSolveUPFError:
      return ("NetSolve: impossible to compile UPF");
      break;
    case NetSolveCannotBind:
      return ("NetSolve: impossible to bind to port");
      break;
    case NetSolveSystemError:
      return ("NetSolve: system error");
      break;
    case NetSolveUnknownError:
      return ("NetSolve: unknown error");
      break;
    case NetSolveSetNetSolveArch:
      return ("NetSolve: NETSOLVE_ARCH not set");
      break;
    case NetSolveSetNetSolveAgent:
      return ("NetSolve: NETSOLVE_AGENT not set");
      break;
    case NetSolveSetNetSolveRoot:
      return ("NetSolve: NETSOLVE_ROOT not set");
    case NetSolveInternalError:
      return ("NetSolve: internal error");
      break;
    case NetSolveMismatch:
      return ("NetSolve: inconsistent object transfers");
      break;
    case NetSolveUnknownHost:
      return ("NetSolve: Unknown host");
      break;
    case NetSolveInvalidUPFFilename:
      return ("NetSolve: invalid upf filename");
      break;
    case NetSolveNetworkError:
      return ("NetSolve: network error");
      break;
    case NetSolveUnknownProblem:
      return ("NetSolve: unknown problem");
      break;
    case NetSolveProtocolError:
      return ("NetSolve: protocol error");
      break;
    case NetSolveNoServer:
      return ("NetSolve: no available server");
      break;
    case NetSolveBadProblemSpecification:
      return ("NetSolve: bad problem input/output");
      break;
    case NetSolveNotAllowed:
      return ("NetSolve: not allowed");
      break;
    case NetSolveBadValues:
      return ("NetSolve: bad input values");
      break;
    case NetSolveDimensionMismatch:
      return ("NetSolve: dimension mismatch");
      break;
    case NetSolveNoSolution:
      return ("NetSolve: no solution");
      break;
    case NetSolveNotReady:
      return ("NetSolve: not ready");
      break;
    case NetSolveInvalidRequestID:
      return ("NetSolve: invalid request ID");
      break;
    case NetSolveBadProblemName:
      return ("NetSolve: invalid problem name");
      break;
    case NetSolveInvalidMajor:
      return ("NetSolve: invalid major specification");
      break;
    case NetSolveTooManyPendingRequests:
      return ("NetSolve: too many pending requests");
      break;
    case NetSolveFileError:
      return ("NetSolve: file I/O error");
      break;
    case NetSolveUnknownDataFormat:
      return ("NetSolve: unknown machine type");
      break;
    case NetSolveTimedOut:
      return ("NetSolve: operation timed out");
      break;
    default:
      return ("NetSolve: unknown error");
      break;
  }
}
