/****
CFCallFrame
*
*
* This class controls the graphical user interface for the fourth
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class CFCallFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  Panel statusPanel;
  GridBagLayout callSeqGridbag;
  GridBagConstraints callSeqConstraints;

  int original_height;
  int original_width;

  Panel callSeqPanel;
  Choice[] callChoice;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;


  public CFCallFrame(PDGUI f) {
  
    super("Calling Sequence for C and Fortran");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    errorPanel.setLayout(new GridLayout(2,1));
    errorPanel.add(errorMessage);

    statusPanel = new Panel();
    statusPanel.setLayout(new BorderLayout(1,5));

    Label titleLabel = new Label
	("The 3 columns represent: Original position, Variables and New positon respectively");
    titleLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE-5));
    titleLabel.setForeground(Color.black);

    callSeqPanel = new Panel();
    callSeqGridbag = new GridBagLayout();
    callSeqConstraints = new GridBagConstraints();
    callSeqPanel.setLayout(callSeqGridbag);
    callSeqConstraints.anchor = GridBagConstraints.NORTH; 
    callSeqConstraints.insets = new Insets(10, 10, 10, 10);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(statusPanel, c);
    topPanel.add(statusPanel);

    gridbag.setConstraints(titleLabel, c);
    topPanel.add(titleLabel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(callSeqPanel, c);
    topPanel.add(callSeqPanel);

    this.pack();
    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;

  }

  public void addStatus(){
   statusPanel.removeAll();
   statusPanel.add("North",parent.statusObject.statusLabel);
   statusPanel.add("South",parent.statusObject.statusArea);
  }

  public void updateFrame(){
  int i, j, argIndex;
  int verticalCount, verticalIndex;
  String variableString;

    statusPanel.add("North",parent.statusObject.statusLabel);
    statusPanel.add("South",parent.statusObject.statusArea);

    callSeqPanel.removeAll();

    GridBagLayout callGridbag = new GridBagLayout();
    GridBagConstraints callConstraints = new GridBagConstraints();
    callConstraints.ipadx  = 10;


    if(parent.problemDescription.argCount >= 10)
      verticalCount = parent.problemDescription.argCount/10 + 1;
    else
      verticalCount = 1;

    Panel[] verticalPanel = new Panel[verticalCount];
    GridBagLayout[] verticalGridbag = new GridBagLayout[verticalCount];
    GridBagConstraints[] verticalConstraints = new GridBagConstraints
						       [verticalCount];

    callChoice = new Choice[parent.problemDescription.argCount];
    for(i=0;i<parent.problemDescription.argCount;i++){
      callChoice[i] = new Choice();
      callChoice[i].setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
      callChoice[i].setBackground(BUTTON_COLOR);
      for(j=0;j<parent.problemDescription.argCount;j++){
        Integer tempInt = new Integer
			(parent.problemDescription.callSequence[j]);
        callChoice[i].add(tempInt.toString());
      }
    }
    
    verticalIndex = -1;

    for(argIndex=0; argIndex < parent.problemDescription.argCount; argIndex++){

      if(argIndex % 10 == 0){
        if(argIndex != 0){
        /* add the previous vertical panel */
          callSeqGridbag.setConstraints
	    (verticalPanel[verticalIndex], callSeqConstraints);
          callSeqPanel.add(verticalPanel[verticalIndex]);
        }
        /* create a new vertical panel */
        verticalIndex++;
        verticalPanel[verticalIndex] = new Panel();
        verticalGridbag[verticalIndex] = new GridBagLayout();
        verticalConstraints[verticalIndex] = new GridBagConstraints();
	verticalPanel[verticalIndex].setLayout
		(verticalGridbag[verticalIndex]);
  
      }
      Integer tempInt = new Integer
	(parent.problemDescription.callSequence[argIndex]);
      Label originalLabel = new Label(tempInt.toString());
      originalLabel.setFont(new Font("Helvetica",Font.BOLD,
			LABEL_FONT_SIZE-5));
      originalLabel.setForeground(Color.black);

      variableString = parent.problemDescription.argmts.userArgs[argIndex];
      Label variableLabel = new Label(variableString);
      variableLabel.setFont(new Font("Helvetica",Font.BOLD,
		LABEL_FONT_SIZE-5));
      variableLabel.setForeground(Color.black);

      callChoice[argIndex].select(argIndex);

      verticalConstraints[verticalIndex].gridwidth = 1;
      verticalConstraints[verticalIndex].anchor = GridBagConstraints.WEST;
      verticalGridbag[verticalIndex].setConstraints(originalLabel, 
			verticalConstraints[verticalIndex]);
      verticalPanel[verticalIndex].add(originalLabel);

      verticalGridbag[verticalIndex].setConstraints(variableLabel, 
		verticalConstraints[verticalIndex]);
      verticalPanel[verticalIndex].add(variableLabel);

      verticalConstraints[verticalIndex].gridwidth = 
			GridBagConstraints.REMAINDER;
      verticalGridbag[verticalIndex].setConstraints(callChoice[argIndex], 
	verticalConstraints[verticalIndex]);
      verticalPanel[verticalIndex].add(callChoice[argIndex]);

    }

    /* add the last vertical panel */
    callSeqGridbag.setConstraints
	(verticalPanel[verticalIndex], callSeqConstraints);
    callSeqPanel.add(verticalPanel[verticalIndex]);


    int argSize = 0;
    argSize = 10;

    this.setSize(original_width + 30*verticalCount,
        original_height+ 15 + 40*argSize); 
    this.validateTree();
  }

  public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
  int i;

    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      if (errorMessage.getText().length() == 0)
      {
        this.setVisible(false);
        parent.matlabCallFrame.addStatus();
        parent.matlabCallFrame.setVisible(true);
      }
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        this.setVisible(false);
        parent.codeFrame.initializeTextArea();
        parent.codeFrame.setVisible(true);
        parent.toolBox.updateFrame();
        parent.toolBox.setVisible(true);
        parent.functionFrame.setVisible(true);
      }
    }
  }

  void pushData(){
  int i;
  String s;
  Integer tempInt;

    for(i=0;i<parent.problemDescription.argCount;i++){
      s = callChoice[i].getSelectedItem();
      tempInt = new Integer(s);
      parent.problemDescription.callSequence[i] = tempInt.intValue();
    }
  }

  String checkData(){
  int i, j, max_val, flag;
  String[] s1;
  String max_s1, s2, s3, s, tempString;
  Integer tempInt;

    max_s1 = "";
    s1 = new String[parent.problemDescription.argCount];
    for(i=0;i<parent.problemDescription.argCount;i++){
      s1[i] = callChoice[i].getSelectedItem();
      if(i==0){
        max_s1 = s1[i];
      }
      else{
        if(s1[i].compareTo(max_s1) > 0){
          max_s1 = s1[i];
        }
      }
    }

    tempInt = new Integer(max_s1);
    max_val = tempInt.intValue();

    s = "";
    for(i=1; i< max_val; i++){
      s2 = s.valueOf(i);
      flag = 0;
      for(j=0; j<parent.problemDescription.argCount;j++){
        if(s2.compareTo(s1[j]) == 0){
          flag = 1;
          break;
        }
      }
      if(flag == 0){
        tempString = "Argument " + i + " not specified\n";
        return tempString;
      }
    }

    for(i=0; i < parent.problemDescription.argCount; i++){
      for(j=i; j < parent.problemDescription.argCount; j++){
        if(s1[i].compareTo(s1[j]) == 0){
          s2 = parent.problemDescription.argmts.argTypes[i];
          s3 = parent.problemDescription.argmts.argTypes[j];
          if(s2.compareTo(s3) != 0){
            tempString = "Incompatible types for argument " + s1[i];
            return tempString;
          }
        }
      }
    }

    return("");
  }
         
  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("cfCallHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      errorMessage.setText("");
    }
  }


  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    int i;
    Object source = e.getSource();
    String s;
    Integer tempInt;

    if (source == nextButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
    if (source == prevButton)
    {
      for(i=0;i<parent.problemDescription.argCount;i++){
        s = callChoice[i].getSelectedItem();
        tempInt = new Integer(s);
        parent.problemDescription.callSequence[i] = tempInt.intValue();
      }
    }
  }
  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}

