/****
CodeFrame
*
*
* This class controls the graphical user interface for the code
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class CodeFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  Panel statusPanel;
  public TextArea outputArea;

  int original_height;
  int original_width;

  private static final int LABEL_FONT_SIZE = 14;


  public CodeFrame(PDGUI f) {

    super("Code Section");

    /* Initialize */
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    statusPanel = new Panel();
    statusPanel.setLayout(new BorderLayout(1,5));

    outputArea = new TextArea("", 32, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(statusPanel, c);
    topPanel.add(statusPanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(outputArea, c);
    topPanel.add(outputArea);

    this.pack();
    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;
  }  

  public void addStatus(){
   statusPanel.removeAll();
   statusPanel.add("North",parent.statusObject.statusLabel);
   statusPanel.add("South",parent.statusObject.statusArea);
  }

   public void initializeTextArea() {
   int pos;

    statusPanel.add("North",parent.statusObject.statusLabel);
    statusPanel.add("South",parent.statusObject.statusArea);

    this.setSize(original_width, original_height+ 150);
    this.validateTree();

    outputArea.selectAll();
    outputArea.replaceRange("", outputArea.getSelectionStart(), 
                                outputArea.getSelectionEnd());

    outputArea.append("START CODE\n");
    outputArea.append("/**\n");
    outputArea.append(" write the main() function of the c code here \n");
    outputArea.append("for e.g., if the main() function looks like \n");
    outputArea.append("main() \n");
    outputArea.append("{ \n");
    outputArea.append("    printf(\"hello world\");\n");
    outputArea.append("}\n");
    outputArea.append("write \n");
    outputArea.append("    printf(\"hello world\");\n");
    outputArea.append("in this area \n");
    outputArea.append("Your code would contain local variables and toolbox variables\n");
    outputArea.append("for e.g. \n");
    outputArea.append("int m;\n");
    outputArea.append("m = 3;\n");
    outputArea.append("A.rows = m;\n");
    outputArea.append("In this code, m is a local variable and A.rows is a toolbox variable\n");
    outputArea.append("To get the toolbox variables, click on the corresponding buttons in the tool box window \n");
    outputArea.append("**/\n");
    outputArea.append("\n\n\n");
    outputArea.append("END CODE\n");
    outputArea.append("\n");

    pos = outputArea.getCaretPosition();
    outputArea.setCaretPosition(pos-12);
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
  int i;

    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      this.setVisible(false);
      parent.cfCallFrame.addStatus();
      parent.cfCallFrame.setVisible(true);
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      pushData();
      parent.functionFrame.pushData();
      this.setVisible(false);
      parent.toolBox.setVisible(false);
      parent.functionFrame.setVisible(false);
      parent.misc.setVisible(true);
    }
  }

  void pushData(){
  String outputString;
  int i;

    outputString = outputArea.getText();
    parent.problemDescription.code = 
		outputString.substring(455, outputString.length()-10);

    for(i=0; i< parent.problemDescription.argCount; i++){

      parent.problemDescription.code = 
		replaceString(parent.problemDescription.argmts.userArgs[i],
			parent.problemDescription.argmts.netsolveArgs[i], 
			parent.problemDescription.code);
    }

  }


  String replaceString 
    (String oldString, String newString, String inputString){ 
    String outputString = ""; 
    int i; 
    for (i=0; i<inputString.length(); i++) { 
      if (!(inputString.regionMatches (true, i, oldString, 0, 
			oldString.length()))) 
        outputString += inputString.charAt(i); 
      else { 
        outputString += newString; 
        i += oldString.length()-1; 
      }
    } 
    return outputString; 
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("codeHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}
