/**
* SecondFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

public class ComplexityFrame extends OkayFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  private Miscellaneous parent;
  private boolean inApplet;

  private TextField aField;
  private TextField bField;
  private TextArea descriptionArea;
  GridBagLayout gridbag;
  GridBagConstraints c;

  private static final int LABEL_FONT_SIZE    = 14;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public ComplexityFrame(Miscellaneous f) {
  
    super("Complexity");
    this.parent = f;
    this.inApplet = parent.inApplet;
    
    /* Take care of the top panel */
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    bottomPanel.add(errorMessage, "North");

    Label descriptionLabel = new Label("Description:");
    descriptionLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    descriptionArea = new TextArea("", 8,  40, TextArea.SCROLLBARS_NONE);
    descriptionArea.setFont(new Font("Helvetica",Font.PLAIN, 12));
    descriptionArea.append("Complexity specifies that given the size\n");
    descriptionArea.append("of the problem, say n, the asymptotic\n");
    descriptionArea.append("complexity, say C, of the problem in\n");
    descriptionArea.append("floating point operations is\n");
    descriptionArea.append("        C = a x n^b\n");
    descriptionArea.setEditable(false);

    Panel entriesPanel = new Panel();
    entriesPanel.setLayout(new GridLayout(1,4));

    Label aLabel = new Label("a:");
    aLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    aField = new TextField(2);
    aField.setEditable(true);

    Label dummyLabel = new Label("  ");
    dummyLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    Label bLabel = new Label("b:");
    bLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    bField = new TextField(2);
    bField.setEditable(true);

    entriesPanel.add(aLabel);
    entriesPanel.add(aField);
    entriesPanel.add(dummyLabel);
    entriesPanel.add(bLabel);
    entriesPanel.add(bField);

    /*c.weightx = 1.0; */
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(descriptionLabel, c);
    topPanel.add(descriptionLabel);
    gridbag.setConstraints(descriptionArea, c);
    topPanel.add(descriptionArea);
    gridbag.setConstraints(entriesPanel, c);
    topPanel.add(entriesPanel);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "quit" button
       pushData();
       this.setVisible(false);
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
     String s = aField.getText();
     if(s.length() != 0){
       Integer a = new Integer(s);
       parent.parent.problemDescription.a = a.intValue();
     }
     String s1 = bField.getText();
     if(s1.length() != 0){
       Integer b = new Integer(s);
       parent.parent.problemDescription.b = b.intValue();
     }
   }

  /**
   *
   * Check the problem name, problem path and problem description
   *
   */

   public String checkData()
   {
   int i;
     String aString = aField.getText();
     int aLength = aString.length();
     for(i=0;i<aLength;i++){
       if(!(Character.isDigit(aString.charAt(i))))
         return new String("Enter integer for a");
     }

     String bString = bField.getText();
     int bLength = bString.length();
     for(i=0;i<bLength;i++){
       if(!(Character.isDigit(bString.charAt(i))))
         return new String("Enter integer for b");
     }

     return new String("");
   }



  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }



}

