/**
* SecondFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

public class CustomFrame extends OkayFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public Miscellaneous parent;
  public boolean inApplet;

  private Choice customChoice;
  private TextArea descriptionArea;
  protected CustomFrame1 customFrame1;
  GridBagLayout gridbag;
  GridBagConstraints c;


  private static final int LABEL_FONT_SIZE    = 14;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public CustomFrame(Miscellaneous f) {
  
    super("Custom");
    this.parent = f;
    this.inApplet = parent.inApplet;

    /* Take care of the top panel */
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    Label descriptionLabel = new Label("Description:");
    descriptionLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    descriptionArea = new TextArea("", 8,  40, TextArea.SCROLLBARS_NONE);
    descriptionArea.setFont(new Font("Helvetica",Font.PLAIN, 12));
    descriptionArea.append("NetSolve has a standard implementation of the\n");
    descriptionArea.append("problems. If you want to customize NetSolve,\n");
    descriptionArea.append("type the name of your customized NetSolve\n");
    descriptionArea.setEditable(false);

    Panel entriesPanel = new Panel();
    entriesPanel.setLayout(new GridLayout(2,1));

    customChoice = new Choice();
    customChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    customChoice.setBackground(BUTTON_COLOR);
    customChoice.add("Custom Name");
    customChoice.add("Scalapack");
    customChoice.add("Iterative solver");
    customChoice.add("Other...");

    entriesPanel.add(customChoice);

    /*c.weightx = 1.0; */
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(descriptionLabel, c);
    topPanel.add(descriptionLabel);
    gridbag.setConstraints(descriptionArea, c);
    topPanel.add(descriptionArea);
    gridbag.setConstraints(entriesPanel, c);
    topPanel.add(entriesPanel);

    customChoice.addItemListener(this);
    /* Initialize */
   customFrame1 = new CustomFrame1(this);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "quit" button
       pushData();
       this.setVisible(false);
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
      String source = customChoice.getSelectedItem();
      if(source.compareTo("Other...") != 0) {
        if(source.compareTo("Scalapack") == 0) 
          parent.parent.problemDescription.customName = "SCALAPACK";
        if(source.compareTo("Iterative solver") == 0)
          parent.parent.problemDescription.customName = "ITER_SOLV";
      }

   }

  /**
   *
   * Check the problem name, problem path and problem description
   *
   */

   public String checkData()
   {
     return new String("");
   }



  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
    String source = (String)e.getItem();
    if(source.compareTo("Other...") == 0) {
      customFrame1.setVisible(true);
    }

  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }



}

