/****
DefinitionFrame
*
*
* This class controls the graphical user interface for the second
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;

public class DefinitionFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  private Panel namePanel;
  private Panel inputPanel;
  private Panel outputPanel;

  private TextField nameField;
  private TextField inputField;
  private TextField outputField;

  private static final int LABEL_FONT_SIZE    = 14;

  public DefinitionFrame(PDGUI f) {
  
    super("Problem Parameters");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    errorPanel.setLayout(new GridLayout(2,1));
    errorPanel.add(errorMessage);

    namePanel = new Panel();

    Label nameLabel = new Label("Function name:");
    nameLabel.setForeground(Color.red);
    nameLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    nameField = new TextField(20);
    nameField.setEditable(true);
    Label nameExample = new Label("e.g.: iterative_solver");
    nameExample.setForeground(Color.black);
    nameExample.setFont(new Font("Helvetica",Font.PLAIN,LABEL_FONT_SIZE));

    GridBagLayout gridbag0 = new GridBagLayout();
    GridBagConstraints c0 = new GridBagConstraints();
    namePanel.setLayout(gridbag0);

    c0.gridwidth = GridBagConstraints.REMAINDER;
    c0.anchor = GridBagConstraints.WEST;
    gridbag0.setConstraints(nameLabel, c0);
    namePanel.add(nameLabel);

    c0.gridwidth = GridBagConstraints.REMAINDER;
    c0.anchor = GridBagConstraints.WEST;
    gridbag0.setConstraints(nameField, c0);
    namePanel.add(nameField);

    c0.gridwidth = GridBagConstraints.REMAINDER;
    c0.anchor = GridBagConstraints.WEST;
    gridbag0.setConstraints(nameExample, c0);
    namePanel.add(nameExample);

    inputPanel = new Panel();

    Label inputLabel = new Label("Comma separated inputs");
    inputLabel.setForeground(Color.red);
    inputLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    inputField = new TextField(20);
    inputField.setEditable(true);
    Label inputExample = new Label("e.g.: A, b");
    inputExample.setForeground(Color.black);
    inputExample.setFont (new Font("Helvetica",Font.PLAIN,LABEL_FONT_SIZE));

    GridBagLayout gridbag1 = new GridBagLayout();
    GridBagConstraints c1 = new GridBagConstraints();
    inputPanel.setLayout(gridbag1);

    c1.gridwidth = GridBagConstraints.REMAINDER;
    c1.anchor = GridBagConstraints.WEST;
    gridbag1.setConstraints(inputLabel, c1);
    inputPanel.add(inputLabel);

    c1.gridwidth = GridBagConstraints.REMAINDER;
    c1.anchor = GridBagConstraints.WEST;
    gridbag1.setConstraints(inputField, c1);
    inputPanel.add(inputField);

    c1.gridwidth = GridBagConstraints.REMAINDER;
    c1.anchor = GridBagConstraints.WEST;
    gridbag1.setConstraints(inputExample, c1);
    inputPanel.add(inputExample);

    outputPanel = new Panel();

    Label outputLabel = new Label("Comma separated outputs");
    outputLabel.setForeground(Color.red);
    outputLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    outputField = new TextField(20);
    outputField.setEditable(true);
    Label outputExample = new Label("e.g.: x, y");
    outputExample.setForeground(Color.black);
    outputExample.setFont (new Font("Helvetica",Font.PLAIN,LABEL_FONT_SIZE));

    GridBagLayout gridbag2 = new GridBagLayout();
    GridBagConstraints c2 = new GridBagConstraints();
    outputPanel.setLayout(gridbag2);

    c2.gridwidth = GridBagConstraints.REMAINDER;
    c2.anchor = GridBagConstraints.WEST;
    gridbag2.setConstraints(outputLabel, c2);
    outputPanel.add(outputLabel);

    c2.gridwidth = GridBagConstraints.REMAINDER;
    c2.anchor = GridBagConstraints.WEST;
    gridbag2.setConstraints(outputField, c2);
    outputPanel.add(outputField);

    c2.gridwidth = GridBagConstraints.REMAINDER;
    c2.anchor = GridBagConstraints.WEST;
    gridbag2.setConstraints(outputExample, c2);
    outputPanel.add(outputExample);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(namePanel, c);
    topPanel.add(namePanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(inputPanel, c);
    topPanel.add(inputPanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(outputPanel, c);
    topPanel.add(outputPanel);

    this.pack();
  }

  public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      if (errorMessage.getText().length() == 0)
      {
        this.setVisible(false);
        parent.descriptionFrame.setVisible(true);
      }
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        this.setVisible(false);
        parent.inoutFrame.initializeInputs();
        parent.inoutFrame.initializeOutputs();
        parent.inoutFrame.updateStatusArea();
        parent.inoutFrame.setVisible(true);
      }
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
     parent.problemDescription.name = nameField.getText();
     pushInputs();
     pushOutputs();

     return;
   }

  public void pushInputs(){
    String inputs, sub;
    int startIndex, endIndex, inputIndex, i;
    int flag, orig_size;

    inputs = inputField.getText();
    inputs.trim();
   
    inputIndex = 0;
    startIndex = 0;
    while((endIndex = inputs.indexOf(",",startIndex)) != -1){
      sub = inputs.substring(startIndex,endIndex);
      sub.trim();

      /* befor inserting, check if the object is already there */
      flag = 0;
      for(i=0;i<parent.problemDescription.inputObjects.size();i++){
        Objects tempObj = (Objects)(parent.problemDescription.
                                      inputObjects.elementAt(i));
        if(sub.compareTo(tempObj.name) == 0){
          /* input object found */
          if(i != inputIndex){
            /* the button is not in the same position */
            parent.problemDescription.inputObjects.removeElementAt(i);
            parent.problemDescription.inputObjects.insertElementAt
				(tempObj, inputIndex);
          }
          flag = 1;
          break;
        }
      }
      if(flag == 0){
      /* the button is not there. add a button */
        parent.problemDescription.inputObjects.insertElementAt
				(new Objects(sub), inputIndex);
      }
 
      startIndex = endIndex + 1;
      inputIndex++;
    }

    endIndex = inputs.length();
    sub = inputs.substring(startIndex,endIndex);
    sub.trim();

    /* befor inserting, check if the object is already there */
    flag = 0;
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                    inputObjects.elementAt(i));
      if(sub.compareTo(tempObj.name) == 0){
        /* input object found */
        if(i != inputIndex){
          /* the button is not in the same position */
          parent.problemDescription.inputObjects.removeElementAt(i);
          parent.problemDescription.inputObjects.insertElementAt
		(tempObj, inputIndex);
        }
        flag = 1;
        break;
      }
    }
    if(flag == 0){
    /* the object is not there. add an object*/
      parent.problemDescription.inputObjects.insertElementAt
				(new Objects(sub),inputIndex);
    }

    inputIndex++;

   /* remove some old objects */
    orig_size = parent.problemDescription.inputObjects.size();
    for(i=inputIndex;i<orig_size;i++){
      parent.problemDescription.inputObjects.removeElementAt(inputIndex);
    }
  }

  public void pushOutputs(){
    String outputs, sub;
    int startIndex, endIndex, outputIndex, i;
    int flag, orig_size;

    outputs = outputField.getText();
    outputs.trim();
   
    outputIndex = 0;
    startIndex = 0;
    while((endIndex = outputs.indexOf(",",startIndex)) != -1){
      sub = outputs.substring(startIndex,endIndex);
      sub.trim();

      /* befor inserting, check if the object is already there */
      flag = 0;
      for(i=0;i<parent.problemDescription.outputObjects.size();i++){
        Objects tempObj = (Objects)(parent.problemDescription.
                                      outputObjects.elementAt(i));
        if(sub.compareTo(tempObj.name) == 0){
          /* output object found */
          if(i != outputIndex){
            /* the button is not in the same position */
            parent.problemDescription.outputObjects.removeElementAt(i);
            parent.problemDescription.outputObjects.insertElementAt
				(tempObj, outputIndex);
          }
          flag = 1;
          break;
        }
      }
      if(flag == 0){
      /* the button is not there. add a button */
        parent.problemDescription.outputObjects.insertElementAt
				(new Objects(sub), outputIndex);
      }
 
      startIndex = endIndex + 1;
      outputIndex++;
    }

    endIndex = outputs.length();
    sub = outputs.substring(startIndex,endIndex);
    sub.trim();

    /* befor inserting, check if the object is already there */
    flag = 0;
    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                    outputObjects.elementAt(i));
      if(sub.compareTo(tempObj.name) == 0){
        /* output object found */
        if(i != outputIndex){
          /* the button is not in the same position */
          parent.problemDescription.outputObjects.removeElementAt(i);
          parent.problemDescription.outputObjects.insertElementAt
		(tempObj, outputIndex);
        }
        flag = 1;
        break;
      }
    }
    if(flag == 0){
    /* the object is not there. add an object*/
      parent.problemDescription.outputObjects.insertElementAt
				(new Objects(sub),outputIndex);
    }

    outputIndex++;

   /* remove some old objects */
    orig_size = parent.problemDescription.outputObjects.size();
    for(i=outputIndex;i<orig_size;i++){
      parent.problemDescription.outputObjects.removeElementAt(outputIndex);
    }
  }

  public String checkData(){
  String s;

     s= nameField.getText();
     if(s.length() == 0)
       return new String("Missing function name");

     s = checkInputs();
     if(s.length() != 0)
       return s;

     s = checkOutputs();
     if(s.length() != 0)
       return s;

     return new String("");
   }     

  public String checkInputs(){
    String inputs, sub;
    int startIndex, endIndex;

    inputs = inputField.getText();
    inputs.trim();
   
    startIndex = 0;
    while((endIndex = inputs.indexOf(",",startIndex)) != -1){
      sub = inputs.substring(startIndex,endIndex);
      sub.trim();
      if(sub.length() == 0)
        return("Inputs cannot be empty");
      startIndex = endIndex + 1;
    }

    endIndex = inputs.length();
    sub = inputs.substring(startIndex,endIndex);
    sub.trim();
    if(sub.length() == 0)
      return("Inputs cannot be empty");

    return new String("");
  }

  public String checkOutputs(){
    String outputs, sub;
    int startIndex, endIndex;

    outputs = outputField.getText();
    outputs.trim();
   
    startIndex = 0;
    while((endIndex = outputs.indexOf(",",startIndex)) != -1){
      sub = outputs.substring(startIndex,endIndex);
      sub.trim();
      if(sub.length() == 0)
        return("Outputs cannot be empty");
      startIndex = endIndex + 1;
    }

    endIndex = outputs.length();
    sub = outputs.substring(startIndex,endIndex);
    sub.trim();
    if(sub.length() == 0)
      return("Outputs cannot be empty");

    return new String("");
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("definitionHelp");
      parent.helpFrame.setVisible(true);
    }

  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      errorMessage.setText("");
    }
  }


  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }
  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}
