/****
DescriptionFrame
*
*
* This class controls the graphical user interface for the Initial
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;

public class DescriptionFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  private TextArea descriptionArea;

  private static final int LABEL_FONT_SIZE    = 14;

  public DescriptionFrame(PDGUI f) {
  
    super("Problem Description");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,1));
    navigatePanel.add("1", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    Panel descriptionPanel = new Panel();
    descriptionPanel.setLayout(new BorderLayout(1,5));

    Label descriptionLabel = new Label("Problem description:");
    descriptionLabel.setForeground(Color.red);
    descriptionLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    descriptionArea = new TextArea("/** This is the default description **/\n",
				10,40,TextArea.SCROLLBARS_VERTICAL_ONLY);
    descriptionArea.setEditable(true);

    descriptionPanel.add("North",descriptionLabel);
    descriptionPanel.add("South",descriptionArea);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(descriptionPanel, c);
    topPanel.add(descriptionPanel);

    this.pack();
  }

  public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        this.setVisible(false);
        parent.definitionFrame.setVisible(true);
      }
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
     parent.problemDescription.description = descriptionArea.getText();

     return;
   }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("descriptionHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }
  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}
