/**
* SecondFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class FileFrame extends OkayFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  private Miscellaneous parent;
  public boolean inApplet;

  private TextField nameField;
  GridBagLayout gridbag;
  GridBagConstraints c;

  private static final int LABEL_FONT_SIZE    = 14;

  DoneFrame doneFrame;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public FileFrame(Miscellaneous f) {
  
    super("FileFrame");
    this.parent = f;
    this.inApplet = parent.parent.inApplet;

    /* Take care of the top panel */
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    Label fileNameLabel = new Label("FileName:");
    fileNameLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    nameField = new TextField(20);
    nameField.setEditable(true);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(fileNameLabel, c);
    topPanel.add(fileNameLabel);
    gridbag.setConstraints(nameField, c);
    topPanel.add(nameField);

    this.pack();

   // initialize
   doneFrame = new DoneFrame(this);
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "quit" button
      // file should be saved here
      if (errorMessage.getText().length() == 0) {
        try{ 
          this.writeToFile();
          doneFrame.updateFrame(nameField.getText());
          doneFrame.setVisible(true);
        }
        catch(IOException e) {
          errorMessage.setText("Error writing to file:  " +
                                  nameField.getText());
        }
      }
       this.setVisible(false);
    }
  }

  public void writeToFile() throws IOException {
     String coreFile;
     String data;
     FileWriter outFileAccess;

     outFileAccess = new FileWriter(nameField.getText());

     outFileAccess.write(parent.parent.problemDescription.outputString);
     outFileAccess.close();
     
  }

   /**
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }



}

