/****
FunctionFrame
*
*
* This class controls the graphical user interface for the function
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class FunctionFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  private TextField functionField;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;

  public FunctionFrame(PDGUI f) {
  
    super("Function Names");
    this.parent = f;
    this.inApplet = parent.inApplet;

    Panel functionPanel = new Panel();
    functionPanel.setLayout(new BorderLayout(1,5));

    Label functionLabel = new Label("Functions used in the code section(separated by commas)");
    functionLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    functionLabel.setForeground(Color.red);
    functionField = new TextField(20);
    functionField.setEditable(true);

    functionPanel.add("North",functionLabel);
    functionPanel.add("South",functionField);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(functionPanel, c);
    topPanel.add(functionPanel);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
  }

  /**
   *
   * update the problem description
   *
   */

  public void pushData(){
    pushFunctions();
  }

  public void pushFunctions(){
    String functions, sub;
    int startIndex, endIndex, i;
    int flag, orig_size;

    functions = functionField.getText();
    functions.trim();
   
    startIndex = 0;
    while((endIndex = functions.indexOf(",",startIndex)) != -1){
      sub = functions.substring(startIndex,endIndex);
      sub.trim();

      if(sub.length() != 0)
        parent.problemDescription.functions.insert(sub);
 
      startIndex = endIndex + 1;
    }

    endIndex = functions.length();
    sub = functions.substring(startIndex,endIndex);
    sub.trim();

    if(sub.length() != 0)
      parent.problemDescription.functions.insert(sub);

   }

  /**
   *
   * Check the problem name, problem path and problem description
   *
   */

   public String checkData()
   {
     String s;

     s = checkFunctions();
     if (s.length() != 0)
       return s;

     return new String("");
   }

  /**
   *
   * Check the functions
   *
   */

   public String checkFunctions()
   {
    String functions, sub;
    int startIndex, endIndex;

    functions = functionField.getText();
    functions.trim();
   
    startIndex = 0;
    while((endIndex = functions.indexOf(",",startIndex)) != -1){
      sub = functions.substring(startIndex,endIndex);
      sub.trim();
      if(sub.length() == 0)
        return("Functions cannot be empty");
      checkFunctionName(sub);
      startIndex = endIndex + 1;
    }

    endIndex = functions.length();
    sub = functions.substring(startIndex,endIndex);
    sub.trim();
    if(sub.length() == 0)
      return("Functions cannot be empty");
    checkFunctionName(sub);

    return new String("");
   }

  /**
   *
   * Check a function Name
   *
   */

   public String checkFunctionName(String s)
   {
     int i;
     s = s.trim();

     if (Character.isLetter(s.charAt(0)) == false)
       return ("'"+s+"' should start with a letter");

     for  (i=1;i<s.length();i++) {
       if ((Character.isLetter(s.charAt(i)) || (s.charAt(i) =='_') ||
           Character.isDigit(s.charAt(i)) ) == false)
         return ("'"+s+"' invalid function name'");
     }
     return new String("");
   }

  /**
     Does nothing
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }



}

