/**
* HelpFrame
*
* This class controls the graphical user interface for the Help
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class HelpFrame extends OkayFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  private PDGUI parent;
  private boolean inApplet;

  GridBagLayout gridbag;
  GridBagConstraints c;

  int original_height;
  int original_width;

  private static final int LABEL_FONT_SIZE    = 14;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public HelpFrame(PDGUI f) {
  
    super("Help");
    this.parent = f;
    this.inApplet = parent.inApplet;
    
    /* Take care of the top panel */
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    this.pack();
    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;
  }  

  void updateFrame(String file){
   
    try{ 
      this.readFromFile(file);
    }
    catch(IOException e) {
      System.err.println("Error reading from file");
    }
  }

  void readFromFile(String file) throws IOException{
  String outString;
  String line;
  int lines;
  int maxLength, tempLength;

    FileReader fin = new FileReader(file);
    BufferedReader bin = new BufferedReader(fin);

    maxLength = 0;
    lines = 0;
    outString = "";

    while((line = bin.readLine()) != null){
      outString = outString + line +"\n";
      tempLength = line.length();
      if(lines == 0)
        maxLength = tempLength;
      else{
        if(tempLength > maxLength)
          maxLength = tempLength;
      }
      lines++;
    }
    bin.close();

    topPanel.removeAll();

    TextArea descriptionArea = new 
		TextArea("", lines,  maxLength, 
			TextArea.SCROLLBARS_NONE);
    descriptionArea.setFont(new Font("Helvetica",Font.PLAIN, 12));
    descriptionArea.append(outString);
    descriptionArea.setEditable(false);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(descriptionArea, c);
    topPanel.add(descriptionArea);

    this.setSize(original_width+maxLength*7,
        original_height+lines*20);
    this.validateTree();
  }

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "quit" button
       this.setVisible(false);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }



}

