/****
InOutFrame
*
*
* This class controls the graphical user interface for the third
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class InOutFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  Panel statusPanel;
  Panel inputButtonsPanel;
  Panel outputButtonsPanel;
  Vector inputButtons;
  Vector outputButtons;
  Vector inputObjectFrame;
  Vector outputObjectFrame;
 
  GridBagLayout inputButtonsGridbag;
  GridBagLayout outputButtonsGridbag;
  GridBagConstraints inputButtonsConstraints;
  GridBagConstraints outputButtonsConstraints;

  int original_height;
  int original_width;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;


  public InOutFrame(PDGUI f) {
  
    super("Inputs and Outputs");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    errorPanel.setLayout(new GridLayout(2,1));
    errorPanel.add(errorMessage);

    statusPanel = new Panel();
    statusPanel.setLayout(new BorderLayout(1,5));

    statusPanel.add("North",parent.statusObject.statusLabel);
    statusPanel.add("South",parent.statusObject.statusArea);

    Label inputObjectsLabel = new Label("Inputs:");
    inputObjectsLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    inputObjectsLabel.setForeground(Color.red);
    inputButtonsPanel = new Panel();
    inputButtonsGridbag = new GridBagLayout();
    inputButtonsConstraints = new GridBagConstraints();
    inputButtonsPanel.setLayout(inputButtonsGridbag);

    Label outputObjectsLabel = new Label("Outputs:");
    outputObjectsLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    outputObjectsLabel.setForeground(Color.red);
    outputButtonsPanel = new Panel();
    outputButtonsGridbag = new GridBagLayout();
    outputButtonsConstraints = new GridBagConstraints();
    outputButtonsPanel.setLayout(outputButtonsGridbag);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(statusPanel, c);
    topPanel.add(statusPanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(inputObjectsLabel, c);
    topPanel.add(inputObjectsLabel);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(inputButtonsPanel, c);
    topPanel.add(inputButtonsPanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(outputObjectsLabel, c);
    topPanel.add(outputObjectsLabel);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(outputButtonsPanel, c);
    topPanel.add(outputButtonsPanel);

    /* initialize */
    inputButtons = new Vector();
    inputObjectFrame = new Vector();
    outputButtons = new Vector();
    outputObjectFrame = new Vector();

    this.pack();
    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;

  }

  public void addStatus(){
   statusPanel.removeAll();
   statusPanel.add("North",parent.statusObject.statusLabel);
   statusPanel.add("South",parent.statusObject.statusArea);
  }

  public void updateStatusArea(){
  String status;
  int i;

   statusPanel.removeAll();
   statusPanel.add("North",parent.statusObject.statusLabel);
   statusPanel.add("South",parent.statusObject.statusArea);

    parent.statusObject.statusArea.setText("");
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                      inputObjects.elementAt(i));
      int temp = i+1;
      status = "Input " + temp + "    " + tempObj.name + "    " +
                    tempObj.dataType + tempObj.objectType + "\n";

      parent.statusObject.statusArea.append(status);

      /* change the button foreground, if necessary */
      if(tempObj.dataType.length() != 0 && tempObj.objectType.length() != 0
         && tempObj.description.length() != 0)
        ((Button)(inputButtons.elementAt(i))).setForeground(Color.white);

    }

    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                      outputObjects.elementAt(i));
      int temp = i+1;
      status = "Output " + temp + "    " + tempObj.name + "    " +
                    tempObj.dataType + tempObj.objectType + "\n";

      parent.statusObject.statusArea.append(status);

      /* change the button foreground, if necessary */
      if(tempObj.dataType.length() != 0 && tempObj.objectType.length() != 0
         && tempObj.description.length() != 0)
        ((Button)(outputButtons.elementAt(i))).setForeground(Color.white);
    }
  }

  public void initializeInputs(){
  int i, rowNumber;


    addInputButtons();

    inputButtonsPanel.removeAll();

    for(i=0; i<inputButtons.size(); i++){

      rowNumber = i/8;
      Button tempButton = (Button)(inputButtons.elementAt(i));
      if(rowNumber == inputButtonsPanel.getComponentCount()){
      /* need to add a new Panel */
        Panel tempRowPanel = new Panel();
        tempRowPanel.add(tempButton);
        inputButtonsConstraints.gridwidth = 
	  GridBagConstraints.REMAINDER;
        inputButtonsConstraints.anchor = GridBagConstraints.WEST;
        inputButtonsGridbag.setConstraints(tempRowPanel, 
	  inputButtonsConstraints);
        inputButtonsPanel.add(tempRowPanel);
      }
      else{
      /* there is already a panel for this row */
        Panel tempRowPanel = (Panel)inputButtonsPanel.
					getComponent(rowNumber);
        tempRowPanel.add(tempButton);
      }
    }
    this.setSize(original_width,
        original_height+30*(inputButtonsPanel.getComponentCount() +
                            outputButtonsPanel.getComponentCount()));
    this.validateTree();

  }

  void addInputButtons(){
  int i, j, flag;
  int orig_size;

  int inputNumber = parent.problemDescription.inputObjects.size();

  for(i=0;i<inputNumber; i++){
    Objects tempObj = (Objects)(parent.problemDescription.
                                    inputObjects.elementAt(i));
    flag = 0;
    for(j=0;j<inputButtons.size();j++){
      Button tempButton = (Button)inputButtons.elementAt(j);
      ObjectFrame tempObjectFrame = (ObjectFrame)inputObjectFrame.elementAt(j);
      // add input object frame
      if(tempObj.name.compareTo(tempButton.getLabel()) == 0 ){
        /* the button is already there */
        if(i != j){
          /* the button is not in the same position */
          inputButtons.removeElementAt(j);
          inputButtons.insertElementAt(tempButton, i);
          inputObjectFrame.removeElementAt(j);
          inputObjectFrame.insertElementAt(tempObjectFrame, i);
        }
        flag = 1;
        break;
      }
    }
    
    if(flag == 0){
    /* the button is not there. add a button */
      Button tempButton = new Button(tempObj.name);
      ObjectFrame tempObjectFrame = new ObjectFrame
					(this, tempObj.name, "Input");
      tempButton.setFont(new Font("Helvetica",Font.BOLD,
             BUTTON_FONT_SIZE));
      tempButton.setBackground(BUTTON_COLOR);
      tempButton.addActionListener(this);
      inputButtons.insertElementAt(tempButton, i);
      inputObjectFrame.insertElementAt(tempObjectFrame, i);
    }
  }

  /* remove some old buttons */
  orig_size = inputButtons.size();
  for(j=i;j<orig_size;j++){
    inputButtons.removeElementAt(i);
    inputObjectFrame.removeElementAt(i);
  }

  }

  public void initializeOutputs(){
  int i, rowNumber;


    addOutputButtons();

    outputButtonsPanel.removeAll();

    for(i=0; i<outputButtons.size(); i++){

      rowNumber = i/8;
      Button tempButton = (Button)(outputButtons.elementAt(i));
      if(rowNumber == outputButtonsPanel.getComponentCount()){
      /* need to add a new Panel */
        Panel tempRowPanel = new Panel();
        tempRowPanel.add(tempButton);
        outputButtonsConstraints.gridwidth = 
	  GridBagConstraints.REMAINDER;
        outputButtonsConstraints.anchor = GridBagConstraints.WEST;
        outputButtonsGridbag.setConstraints(tempRowPanel, 
	  outputButtonsConstraints);
        outputButtonsPanel.add(tempRowPanel);
      }
      else{
      /* there is already a panel for this row */
        Panel tempRowPanel = (Panel)outputButtonsPanel.
					getComponent(rowNumber);
        tempRowPanel.add(tempButton);
      }
    }
    this.setSize(original_width,
        original_height+30*(inputButtonsPanel.getComponentCount() +
                            outputButtonsPanel.getComponentCount()));
    this.validateTree();

  }

  void addOutputButtons(){
 int i, j, flag;
  int orig_size;

  int outputNumber = parent.problemDescription.outputObjects.size();

  for(i=0;i<outputNumber; i++){
    Objects tempObj = (Objects)(parent.problemDescription.
                                    outputObjects.elementAt(i));
    flag = 0;
    for(j=0;j<outputButtons.size();j++){
      Button tempButton = (Button)outputButtons.elementAt(j);
      ObjectFrame tempObjectFrame = 
				(ObjectFrame)outputObjectFrame.elementAt(j);
      // add output object frame
      if(tempObj.name.compareTo(tempButton.getLabel()) == 0 ){
        /* the button is already there */
        if(i != j){
          /* the button is not in the same position */
          outputButtons.removeElementAt(j);
          outputButtons.insertElementAt(tempButton, i);
          outputObjectFrame.removeElementAt(j);
          outputObjectFrame.insertElementAt(tempObjectFrame, i);
        }
        flag = 1;
        break;
      }
    }
    
    if(flag == 0){
    /* the button is not there. add a button */
      Button tempButton = new Button(tempObj.name);
      ObjectFrame tempObjectFrame = new ObjectFrame
					(this, tempObj.name, "Output");
      tempButton.setFont(new Font("Helvetica",Font.BOLD,
             BUTTON_FONT_SIZE));
      tempButton.setBackground(BUTTON_COLOR);
      tempButton.addActionListener(this);
      outputButtons.insertElementAt(tempButton, i);
      outputObjectFrame.insertElementAt(tempObjectFrame, i);
    }
  }

  /* remove some old buttons */
  orig_size = outputButtons.size();
  for(j=i;j<orig_size;j++){
    outputButtons.removeElementAt(i);
    outputObjectFrame.removeElementAt(i);
  }

  }


  public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
  int i;

    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      if (errorMessage.getText().length() == 0)
      {
        this.setVisible(false);
        parent.definitionFrame.setVisible(true);
      }
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        this.setVisible(false);
        parent.matlabCallFrame.updateFrame();
        parent.matlabCallFrame.setVisible(true);
      }
    }
    else{
     // input or output buttons 
     for(i=0;i<inputButtons.size();i++){
       if(source == (Button)(inputButtons.elementAt(i))) {
         ObjectFrame objFrame = (ObjectFrame)(inputObjectFrame.elementAt(i));
         objFrame.setVisible(true);
         break;
       }
     }
     for(i=0;i<outputButtons.size();i++){
       if(source == (Button)(outputButtons.elementAt(i))) {
         ObjectFrame objFrame = (ObjectFrame)(outputObjectFrame.elementAt(i));
         objFrame.setVisible(true);
         break;
       }
     }

   }
  }

  void pushData(){
  int i, j;
  int max= 0, maxFlag=0;
  int argCount, argIndex;
   

    /* initialize input object sequence */
    parent.problemDescription.inputobjectSequence = new int
			[parent.problemDescription.inputObjects.size()];
    for(i=0; i<parent.problemDescription.inputObjects.size(); i++)
      parent.problemDescription.inputobjectSequence[i] = i+1;

    /* initialize output object sequence */
    parent.problemDescription.outputobjectSequence = new int
			[parent.problemDescription.outputObjects.size()];
    for(i=0; i<parent.problemDescription.outputObjects.size(); i++)
      parent.problemDescription.outputobjectSequence[i] = i+1;

    argCount = 0;
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      Objects obj = (Objects)(parent.problemDescription.inputObjects.
                                                elementAt(i));
      if(maxFlag == 0){
        max = obj.callObj.argCount;
        maxFlag = 1;
      }
      if(obj.callObj.argCount > max)
        max = obj.callObj.argCount;

      argCount += obj.callObj.argCount;
    }

    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      Objects obj = (Objects)(parent.problemDescription.outputObjects.
                                                elementAt(i));
      if(maxFlag == 0){
        max = obj.callObj.argCount;
        maxFlag = 1;
      }
      if(obj.callObj.argCount > max)
        max = obj.callObj.argCount;

      argCount += obj.callObj.argCount;
    }

    parent.problemDescription.maxCount = max;
    if(argCount != parent.problemDescription.argCount){
      parent.problemDescription.argCount = argCount;
      parent.problemDescription.callSequence =
			new int[parent.problemDescription.argCount];

      for(i=0;i<parent.problemDescription.argCount;i++)
        parent.problemDescription.callSequence[i] = i+1;

    }
  }

   public String checkData()
   {
     String s;

     s = checkInputObjs();
     if (s.length() != 0)
       return s;

     s = checkOutputObjs();
     if (s.length() != 0)
       return s;

     return new String("");
   }

  public String checkInputObjs(){
  int i;

    for(i=0;i<inputButtons.size();i++){
      if((Color)((Button)(inputButtons.elementAt(i))).getForeground() !=
  		Color.white){
        int temp = i+1;
        Objects tempObj = (Objects)(parent.problemDescription.
                                      inputObjects.elementAt(i));
        return("Input object "+ tempObj.name + " not defined");
      }
    }
    return new String("");
  }

  public String checkOutputObjs(){
  int i;

    for(i=0;i<outputButtons.size();i++){
      if((Color)((Button)(outputButtons.elementAt(i))).getForeground() 
		!= Color.white){
        int temp = i+1;
        Objects tempObj = (Objects)(parent.problemDescription.
                                      outputObjects.elementAt(i));
        return("Output object "+ tempObj.name + " not defined");
      }
    }
    return new String("");
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("inoutHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      errorMessage.setText("");
    }
  }


  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }
  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}
