/**
* SecondFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

public class LibFrame extends IncLib
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  private Miscellaneous parent;
  private boolean inApplet;
  private Label libDirLabel;
  private Label libFilesLabel;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public LibFrame(Miscellaneous f) {
  
    super("Libraries");
    this.parent = f;
    this.inApplet = parent.inApplet;

    libDirLabel = new Label("Library directories");
    libDirLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    libFilesLabel = new Label("Libraries to be included (one per line):");
    libFilesLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    label1Panel.add(libDirLabel);
    label2Panel.add(libFilesLabel);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "quit" button
      if (errorMessage.getText().length() == 0)
      {
       pushData();
       this.setVisible(false);
      }
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
     pushLibDirs();
     pushLibFiles();
   }

   public void pushLibDirs(){
     int i,start,end;
     String s;

     /* clear the functions of the Problem Description */
     parent.parent.problemDescription.libdirs.clear();

     s = area1.getText();

     if (s.endsWith("\n") == false)
       s = s+"\n";

     start = 0;
     for (i=0;i<s.length();i++) {
       if (s.charAt(start) == '\n') {
         start++;
         continue;
       }
       if (s.charAt(i) == '\n') {
         end = i;
         parent.parent.problemDescription.libdirs.insert
                (s.substring(start,end));
         start = i+1;
       }
     }
   }

  public void pushLibFiles(){
     int i,start,end;
     String s;

     /* clear the functions of the Problem Description */
     parent.parent.problemDescription.libs.clear();

     s = area2.getText();

     if (s.endsWith("\n") == false)
       s = s+"\n";

     start = 0;
     for (i=0;i<s.length();i++) {
       if (s.charAt(start) == '\n') {
         start++;
         continue;
       }
       if (s.charAt(i) == '\n') {
         end = i;
         parent.parent.problemDescription.libs.insert
                (s.substring(start,end));
         start = i+1;
       }
     }
   }


  /**
   *
   * Check the problem name, problem path and problem description
   *
   */

   public String checkData()
   {
     String s;

     s = checkLibs();
     if (s.length() != 0)
       return s;

     return new String("");
   }


  /**
   *
   * Check the libraries
   *
   */

   public String checkLibs()
   {
     int i,start,end;
     String s,error;

     /* clear the functions of the Problem Description */
     parent.parent.problemDescription.libs.clear();

     s = area2.getText();

     if (s.endsWith("\n") == false)
       s = s+"\n";

     start = 0;
     for (i=0;i<s.length();i++) {
       if (s.charAt(start) == '\n') {
         start++;
         continue;
       }
       if (s.charAt(i) == '\n') {
         end = i;
         error = checkLibName(s.substring(start,end));
         if (error.length() != 0)
           return error;
         else
         {
           parent.parent.problemDescription.libs.insert(s.substring(start,end));
           start = i+1;
         }
       }
     }
     return new String("");
   }

  /**
   *
   * Checks a lib file
   *
   */

   public String checkLibName(String s)
   {
     int i;
     s = s.trim();

     if(!(s.startsWith("-l")) && !(s.endsWith(".a")))
       return ("'"+s+"' should either start with '-l' or end with '.a'");

     if(s.startsWith("-l") && s.endsWith(".a"))
       return ("'"+s+"' should either start with '-l' or end with '.a'");

     return new String("");

   }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }



}

