/****
MatlabCallFrame
*
*
* This class controls the graphical user interface for the matlabcall
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class MatlabCallFrame extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  Panel statusPanel;
  GridBagLayout callSeqGridbag;
  GridBagConstraints callSeqConstraints;
  GridBagLayout inputGridbag;
  GridBagConstraints inputConstraints;
  GridBagLayout outputGridbag;
  GridBagConstraints outputConstraints;

  Label inputLabel;
  Label outputLabel;

  int original_height;
  int original_width;

  Panel inputPanel;
  Panel outputPanel;
  Panel callSeqPanel;
  Choice[] call1Choice;
  Choice[] call2Choice;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;


  public MatlabCallFrame(PDGUI f) {
  
    super("Calling Sequence for Matlab");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", nextButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    errorPanel.setLayout(new GridLayout(2,1));
    errorPanel.add(errorMessage);

    statusPanel = new Panel();
    statusPanel.setLayout(new BorderLayout(1,5));

    Label titleLabel = new Label
	("The 3 columns represent: Original position, Objects and New positon respectively");
    titleLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE-5));
    titleLabel.setForeground(Color.black);

    inputPanel = new Panel();
    inputGridbag = new GridBagLayout();
    inputConstraints = new GridBagConstraints();
    inputPanel.setLayout(inputGridbag);

    inputLabel = new Label("Input Objects");
    inputLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE));
    inputLabel.setForeground(Color.black);


    outputPanel = new Panel();
    outputGridbag = new GridBagLayout();
    outputConstraints = new GridBagConstraints();
    outputPanel.setLayout(outputGridbag);

    outputLabel = new Label("Output Objects");
    outputLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE));
    outputLabel.setForeground(Color.black);

    callSeqPanel = new Panel();
    callSeqGridbag = new GridBagLayout();
    callSeqConstraints = new GridBagConstraints();
    callSeqPanel.setLayout(callSeqGridbag);

    callSeqConstraints.anchor = GridBagConstraints.NORTHWEST; 
    callSeqGridbag.setConstraints(inputPanel,callSeqConstraints);
    callSeqPanel.add(inputPanel);
    callSeqConstraints.anchor = GridBagConstraints.NORTHWEST; 
    callSeqGridbag.setConstraints(outputPanel,callSeqConstraints);
    callSeqPanel.add(outputPanel);

    /*callSeqConstraints.insets = new Insets(10, 10, 10, 10); */

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(statusPanel, c);
    topPanel.add(statusPanel);

    gridbag.setConstraints(titleLabel, c);
    topPanel.add(titleLabel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(callSeqPanel, c);
    topPanel.add(callSeqPanel);

    this.pack();
    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;

  }

  public void addStatus(){
   statusPanel.removeAll();
   statusPanel.add("North",parent.statusObject.statusLabel);
   statusPanel.add("South",parent.statusObject.statusArea);
  }

  public void updateFrame(){
  int i, j, objIndex;
  int vertical1Count, vertical2Count, verticalIndex;
  String variableString;


    statusPanel.add("North",parent.statusObject.statusLabel);
    statusPanel.add("South",parent.statusObject.statusArea);

    inputPanel.removeAll();
    inputConstraints.gridwidth = GridBagConstraints.REMAINDER;
    inputConstraints.anchor = GridBagConstraints.CENTER;
    inputConstraints.insets = new Insets(10, 10, 10, 10);

    inputGridbag.setConstraints(inputLabel, inputConstraints);
    inputPanel.add(inputLabel);

    inputConstraints.gridwidth = 1;

    outputPanel.removeAll();
    outputConstraints.gridwidth = GridBagConstraints.REMAINDER;
    outputConstraints.anchor = GridBagConstraints.CENTER;
    outputConstraints.insets = new Insets(10, 10, 10, 10);

    outputGridbag.setConstraints(outputLabel, outputConstraints);
    outputPanel.add(outputLabel);

    outputConstraints.gridwidth = 1;

    if(parent.problemDescription.inputObjects.size() >= 10)
      vertical1Count = parent.problemDescription.inputObjects.size()/10 + 1;
    else
      vertical1Count = 1;

    if(parent.problemDescription.inputObjects.size() >= 10)
      vertical2Count = parent.problemDescription.outputObjects.size()/10 + 1;
    else
      vertical2Count = 1;

    Panel[] vertical1Panel = new Panel[vertical1Count];
    GridBagLayout[] vertical1Gridbag = new GridBagLayout[vertical1Count];
    GridBagConstraints[] vertical1Constraints = new GridBagConstraints
						       [vertical1Count];

    Panel[] vertical2Panel = new Panel[vertical2Count];
    GridBagLayout[] vertical2Gridbag = new GridBagLayout[vertical2Count];
    GridBagConstraints[] vertical2Constraints = new GridBagConstraints
						       [vertical2Count];

    call1Choice = new Choice[parent.problemDescription.inputObjects.size()];
    call2Choice = new Choice[parent.problemDescription.outputObjects.size()];

    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      call1Choice[i] = new Choice();
      call1Choice[i].setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
      call1Choice[i].setBackground(BUTTON_COLOR);
      for(j=0;j<parent.problemDescription.inputObjects.size();j++){
        Integer tempInt = new Integer
			(parent.problemDescription.inputobjectSequence[j]);
        call1Choice[i].add(tempInt.toString());
      }
    }

    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      call2Choice[i] = new Choice();
      call2Choice[i].setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
      call2Choice[i].setBackground(BUTTON_COLOR);
      for(j=0;j<parent.problemDescription.outputObjects.size();j++){
        Integer tempInt = new Integer
			(parent.problemDescription.outputobjectSequence[j]);
        call2Choice[i].add(tempInt.toString());
      }
    }

    
    verticalIndex = -1;
    objIndex = 0;
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                      inputObjects.elementAt(i));
      if(objIndex % 10 == 0){
        if(objIndex != 0){
          /* add the previous vertical panel */
            inputGridbag.setConstraints
	  (vertical1Panel[verticalIndex], inputConstraints);
            inputPanel.add(vertical1Panel[verticalIndex]);
        }
        /* create a new vertical panel */
        verticalIndex++;
        vertical1Panel[verticalIndex] = new Panel();
        vertical1Gridbag[verticalIndex] = new GridBagLayout();
        vertical1Constraints[verticalIndex] = new GridBagConstraints();
        vertical1Panel[verticalIndex].setLayout
				(vertical1Gridbag[verticalIndex]);
  
      }

      Integer tempInt = new Integer
		(parent.problemDescription.inputobjectSequence[objIndex]);
      Label originalLabel = new Label(tempInt.toString());
      originalLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE-5));
      originalLabel.setForeground(Color.black);

      variableString = tempObj.name;
      Label variableLabel = new Label(variableString);
      variableLabel.setFont(new Font("Helvetica",Font.BOLD,
			LABEL_FONT_SIZE-5));
      variableLabel.setForeground(Color.black);

      call1Choice[objIndex].select(objIndex);

      vertical1Constraints[verticalIndex].gridwidth = 1;
      vertical1Constraints[verticalIndex].anchor = GridBagConstraints.WEST;
      vertical1Gridbag[verticalIndex].setConstraints(originalLabel, 
					vertical1Constraints[verticalIndex]);
      vertical1Panel[verticalIndex].add(originalLabel);

      vertical1Gridbag[verticalIndex].setConstraints(variableLabel, 
				vertical1Constraints[verticalIndex]);
      vertical1Panel[verticalIndex].add(variableLabel);

      vertical1Constraints[verticalIndex].gridwidth = 
					GridBagConstraints.REMAINDER;
      vertical1Gridbag[verticalIndex].setConstraints(call1Choice[objIndex], 
			vertical1Constraints[verticalIndex]);
      vertical1Panel[verticalIndex].add(call1Choice[objIndex]);

      objIndex++;
    } /* input objects */

    /* add the last vertical panel */
    inputGridbag.setConstraints
	(vertical1Panel[verticalIndex], inputConstraints);
    inputPanel.add(vertical1Panel[verticalIndex]);

    verticalIndex = -1;
    objIndex = 0;
    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      Objects tempObj = (Objects)(parent.problemDescription.
                                      outputObjects.elementAt(i));
      if(objIndex % 10 == 0){
        if(objIndex != 0){
          /* add the previous vertical panel */
            outputGridbag.setConstraints
	  (vertical2Panel[verticalIndex], outputConstraints);
            outputPanel.add(vertical2Panel[verticalIndex]);
        }
        /* create a new vertical panel */
        verticalIndex++;
        vertical2Panel[verticalIndex] = new Panel();
        vertical2Gridbag[verticalIndex] = new GridBagLayout();
        vertical2Constraints[verticalIndex] = new GridBagConstraints();
        vertical2Panel[verticalIndex].setLayout
				(vertical2Gridbag[verticalIndex]);
  
      }

      Integer tempInt = new Integer
		(parent.problemDescription.outputobjectSequence[objIndex]);
      Label originalLabel = new Label(tempInt.toString());
      originalLabel.setFont(new Font("Helvetica",Font.BOLD,
				LABEL_FONT_SIZE-5));
      originalLabel.setForeground(Color.black);

      variableString = tempObj.name;
      Label variableLabel = new Label(variableString);
      variableLabel.setFont(new Font("Helvetica",Font.BOLD,
			LABEL_FONT_SIZE-5));
      variableLabel.setForeground(Color.black);

      call2Choice[objIndex].select(objIndex);

      vertical2Constraints[verticalIndex].gridwidth = 1;
      vertical2Constraints[verticalIndex].anchor = GridBagConstraints.WEST;
      vertical2Gridbag[verticalIndex].setConstraints(originalLabel, 
					vertical2Constraints[verticalIndex]);
      vertical2Panel[verticalIndex].add(originalLabel);

      vertical2Gridbag[verticalIndex].setConstraints(variableLabel, 
				vertical2Constraints[verticalIndex]);
      vertical2Panel[verticalIndex].add(variableLabel);

      vertical2Constraints[verticalIndex].gridwidth = 
					GridBagConstraints.REMAINDER;
      vertical2Gridbag[verticalIndex].setConstraints(call2Choice[objIndex], 
			vertical2Constraints[verticalIndex]);
      vertical2Panel[verticalIndex].add(call2Choice[objIndex]);

      objIndex++;
    } /* output objects */

    /* add the last vertical panel */
    outputGridbag.setConstraints
	(vertical2Panel[verticalIndex], outputConstraints);
    outputPanel.add(vertical2Panel[verticalIndex]);


    int argSize = 0;
    argSize = 10;

    this.setSize(original_width + 30*(vertical1Count + vertical2Count) ,
        original_height+ 50 + 40*argSize); 
    this.validateTree();
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
  int i;

    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      if (errorMessage.getText().length() == 0)
      {
        this.setVisible(false);
        parent.inoutFrame.addStatus();
        parent.inoutFrame.setVisible(true);
      }
    }
    else if( source == nextButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        this.setVisible(false);
        parent.cfCallFrame.updateFrame();
        parent.cfCallFrame.setVisible(true);
      }
    }
  }


  void pushData(){
  int i, j, k, temp, argCount, argIndex;
  String s;
  Integer tempInt;

    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      s = call1Choice[i].getSelectedItem();
      tempInt = new Integer(s);
      parent.problemDescription.inputobjectSequence[i] = tempInt.intValue();
    }
    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      s = call2Choice[i].getSelectedItem();
      tempInt = new Integer(s);
      parent.problemDescription.outputobjectSequence[i] = tempInt.intValue();
    }

    argCount = parent.problemDescription.argCount;

    parent.problemDescription.argmts.userArgs = new String[argCount];
    parent.problemDescription.argmts.netsolveArgs = new String[argCount];
    parent.problemDescription.argmts.argTypes = new String[argCount];

    argIndex = 0;
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){

      j = parent.problemDescription.inputobjectSequence[i];
      temp = j-1;
      Objects obj = (Objects)
		(parent.problemDescription.inputObjects.elementAt(temp));
      for(k=0; k<obj.callObj.argCount; k++){
        parent.problemDescription.argmts.userArgs[argIndex] = 
          obj.name + "." + obj.callObj.argmts[k];

        parent.problemDescription.argmts.netsolveArgs[argIndex] = 
          obj.callObj.mnemonics[k] + "I" + i;

        parent.problemDescription.argmts.argTypes[argIndex] = 
          obj.callObj.type[k];

        argIndex++;
      }

    }

    for(i=0;i<parent.problemDescription.outputObjects.size();i++){

      j = parent.problemDescription.outputobjectSequence[i];
      temp = j-1;
      Objects obj = (Objects)
		(parent.problemDescription.outputObjects.elementAt(j-1));
      for(k=0; k<obj.callObj.argCount; k++){
        parent.problemDescription.argmts.userArgs[argIndex] = 
          obj.name + "." + obj.callObj.argmts[k];

        parent.problemDescription.argmts.netsolveArgs[argIndex] = 
          obj.callObj.mnemonics[k] + "O" + i;

        parent.problemDescription.argmts.argTypes[argIndex] = 
          obj.callObj.type[k];

        argIndex++;
      }

    }


  }

  String checkData(){
  int i, j, max_val, flag;
  String []s1;
  String []s2;
  String s3, s, tempString, max_s1, max_s2;
  Integer tempInt;

    s1 = new String[parent.problemDescription.inputObjects.size()];
    for(i=0;i<parent.problemDescription.inputObjects.size();i++){
      s1[i] = call1Choice[i].getSelectedItem();
      for(j=i+1;j<parent.problemDescription.inputObjects.size();j++){
        s3 = call1Choice[j].getSelectedItem();
        if(s1[i].compareTo(s3) == 0){
          return("2 input objects in the same positon\n");
        }
      }
    }

    s2 = new String[parent.problemDescription.outputObjects.size()];
    for(i=0;i<parent.problemDescription.outputObjects.size();i++){
      s2[i] = call2Choice[i].getSelectedItem();
      for(j=i+1;j<parent.problemDescription.outputObjects.size();j++){
        s3 = call2Choice[j].getSelectedItem();
        if(s2[i].compareTo(s3) == 0){
          return("2 output objects in the same positon\n");
        }
      }
    }

    return("");
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("matlabCallHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == nextButton)
    {
      errorMessage.setText("");
    }
  }


  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    int i;
    Object source = e.getSource();
    String s;
    Integer tempInt;

    if (source == nextButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
    if (source == prevButton)
    {
      for(i=0;i<parent.problemDescription.inputObjects.size();i++){
        s = call1Choice[i].getSelectedItem();
        tempInt = new Integer(s);
        parent.problemDescription.inputobjectSequence[i] = tempInt.intValue();
      }
      for(i=0;i<parent.problemDescription.outputObjects.size();i++){
        s = call2Choice[i].getSelectedItem();
        tempInt = new Integer(s);
        parent.problemDescription.outputobjectSequence[i] = tempInt.intValue();
      }
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}

