/****
Miscellaneous
*
*
* This class controls the graphical user interface for the miscellaneous
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class Miscellaneous extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  protected Button includeButton;
  protected Button libButton;
  public IncludeFrame includeFrame;
  public LibFrame libFrame;

  private Choice languageChoice;
  private Choice majorChoice;
  protected Button complexityButton;
  protected Button customButton;
  public ComplexityFrame complexityFrame;
  public CustomFrame customFrame;

  FileFrame fileFrame;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;


  public Miscellaneous(PDGUI f) {
  
    super("Miscellaneous");
    this.parent = f;
    this.inApplet = parent.inApplet;

    navigatePanel.setLayout(new GridLayout(1,2));
    navigatePanel.add("1", prevButton);
    navigatePanel.add("2", saveButton);

    quitPanel.setLayout(new GridLayout(1,1));
    quitPanel.add("1",quitButton); 

    errorPanel.setLayout(new GridLayout(2,1));
    errorPanel.add(errorMessage);

    includeButton = new Button(" Includes");
    includeButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    includeButton.setBackground(BUTTON_COLOR);

    libButton = new Button("Libraries");
    libButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    libButton.setBackground(BUTTON_COLOR);

    Panel langMajorPanel = new Panel();

    languageChoice = new Choice();
    languageChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    languageChoice.setBackground(BUTTON_COLOR);
    languageChoice.add("Languages");
    languageChoice.add("C");
    languageChoice.add("FORTRAN");

    majorChoice = new Choice();
    majorChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    majorChoice.setBackground(BUTTON_COLOR);
    majorChoice.add("Major");
    majorChoice.add("ROW");
    majorChoice.add("COLUMN");

    langMajorPanel.add(languageChoice);
    langMajorPanel.add(majorChoice);

    complexityButton = new Button("Complexity");
    complexityButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    complexityButton.setBackground(BUTTON_COLOR);

    customButton = new Button("Custom Function?");
    customButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    customButton.setBackground(BUTTON_COLOR);

    c.anchor = GridBagConstraints.CENTER;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.insets = new Insets(10, 0, 0, 10);
    gridbag.setConstraints(includeButton, c);
    topPanel.add(includeButton);

    gridbag.setConstraints(libButton, c);
    topPanel.add(libButton);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(langMajorPanel, c);
    topPanel.add(langMajorPanel);

    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(complexityButton, c);
    topPanel.add(complexityButton);

    c.weighty = 1.0;
    c.insets = new Insets(10, 0, 0, 10);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(customButton, c);
    topPanel.add(customButton);

    languageChoice.addItemListener(this);
    majorChoice.addItemListener(this);
    complexityButton.addActionListener(this);
    customButton.addActionListener(this);

    includeButton.addActionListener(this);
    libButton.addActionListener(this);


    /* initalize */
    includeFrame = new IncludeFrame(this);
    libFrame = new LibFrame(this);
    complexityFrame = new ComplexityFrame(this);
    customFrame = new CustomFrame(this);
    fileFrame = new FileFrame(this);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else if( source == prevButton ) {
      // user clicked the "prev" button
      if (errorMessage.getText().length() == 0)
      {
        this.setVisible(false);
        parent.codeFrame.addStatus();
        parent.codeFrame.setVisible(true);
        parent.toolBox.setVisible(true);
        parent.functionFrame.setVisible(true);
      }
    }
    else if( source == saveButton ) {
      // user clicked the "next" button
      if (errorMessage.getText().length() == 0)
      {
        pushData();
        fileFrame.setVisible(true);
      }
    }
    else if( source == includeButton ) {
      // user clicked the "include" button
         includeFrame.setVisible(true);
    }
    else if( source == libButton ) {
      // user clicked the "libraries" button
         libFrame.setVisible(true);
    }
    else if( source == complexityButton ) {
      // user clicked the "complexity" button
         complexityFrame.setVisible(true);
    }
    else if( source == customButton ) {
      // user clicked the "custom" button
         customFrame.setVisible(true);
    }
  }

  /**
   *
   * update the problem description
   *
   */

   public void pushData()
   {
     parent.problemDescription.language = languageChoice.getSelectedItem();

     parent.problemDescription.major = majorChoice.getSelectedItem();

     doBlock();

     return;
   }

   private int doBlock() {
      ProblemDescription pd;
      LinkedList tmpLL;
      int i, j, firstTime, done;
      String tempString;

      pd = parent.problemDescription;
   
      pd.outputString = pd.outputString + "@PROBLEM " + pd.name + "\n";
      pd.outputString = pd.outputString + "@PATH  /Bubba/bubba/\n";
      pd.outputString = pd.outputString + "@DESCRIPTION\n";
      pd.outputString = pd.outputString + pd.description;
      pd.outputString = pd.outputString + "@LANGUAGE " + pd.language + "\n";
      pd.outputString = pd.outputString + "@MAJOR " + pd.major + "\n";
      pd.outputString = pd.outputString + "@COMPLEXITY " + 
				pd.a + "," + pd.b + "\n";
      if(pd.customName.length() != 0)
        pd.outputString = pd.outputString + "@CUSTOM " + pd.customName + "\n";
      pd.outputString = pd.outputString + "\n";

      for (tmpLL = pd.functions; tmpLL != null; tmpLL = tmpLL.next) {
         if (tmpLL.content != null) {
            pd.outputString = pd.outputString + "@FUNCTION " + 
				(String) tmpLL.content + "\n";
         }
      }
      for (tmpLL = pd.dashis; tmpLL != null; tmpLL = tmpLL.next) {
         if (tmpLL.content != null) {
            pd.outputString = pd.outputString + "@DASHI " + 
				(String) tmpLL.content + "\n";
         }
      }
      for (tmpLL = pd.includes; tmpLL != null; tmpLL = tmpLL.next) {
         if (tmpLL.content != null) {
            pd.outputString = pd.outputString + "@INCLUDE " + 
				(String) tmpLL.content + "\n";
         }
      }
      for (tmpLL = pd.libdirs; tmpLL != null; tmpLL = tmpLL.next) {
         if (tmpLL.content != null) {
            pd.outputString = pd.outputString + "@LIB -L" + 
				(String) tmpLL.content + "\n";
         }
      }
      for (tmpLL = pd.libs; tmpLL != null; tmpLL = tmpLL.next) {
         if (tmpLL.content != null) {
            pd.outputString = pd.outputString + "@LIB " + 
				(String) tmpLL.content + "\n";
         }
      }
      pd.outputString = pd.outputString + "\n";

      pd.outputString = pd.outputString + "@INPUT " + " " + 
				pd.inputObjects.size() + "\n";

      for(i=0;i<pd.inputObjects.size();i++){

        j = pd.inputobjectSequence[i];
        Objects obj = (Objects)(pd.inputObjects.elementAt(j-1));
        pd.outputString = pd.outputString + "@OBJECT " ;
        if(obj.objectType.compareTo("String list") == 0)
          pd.outputString = pd.outputString + "STRINGLIST";
        else if(obj.objectType.compareTo("Packed Files") == 0)
          pd.outputString = pd.outputString + "PACKEDFILES";
        else
          pd.outputString = pd.outputString + obj.objectType.toUpperCase();

        pd.outputString = pd.outputString + " ";

        if(obj.dataType.compareTo("Integer") == 0)
          pd.outputString = pd.outputString + "I";
        else if(obj.dataType.compareTo("Float") == 0)
          pd.outputString = pd.outputString + "S";
        else if(obj.dataType.compareTo("Double") == 0)
          pd.outputString = pd.outputString + "D";
        else if(obj.dataType.compareTo("Char") == 0)
          pd.outputString = pd.outputString + "CHAR";
        else if(obj.dataType.compareTo("Byte") == 0)
          pd.outputString = pd.outputString + "B";
        else if(obj.dataType.compareTo("Simple Complex") == 0)
          pd.outputString = pd.outputString + "C";
        else if(obj.dataType.compareTo("Double Complex") == 0)
          pd.outputString = pd.outputString + "Z";

        pd.outputString = pd.outputString + " " + obj.name + "\n";
         
        pd.outputString = pd.outputString + obj.description;
        pd.outputString = pd.outputString + "\n";
      }
      pd.outputString = pd.outputString + "\n";

      pd.outputString = pd.outputString + "@OUTPUT " + " " + 
				pd.outputObjects.size() + "\n";

      for(i=0;i<pd.outputObjects.size();i++){

        j = pd.outputobjectSequence[i];
        Objects obj = (Objects)(pd.outputObjects.elementAt(j-1));
        pd.outputString = pd.outputString + "@OBJECT " ;
        if(obj.objectType.compareTo("String list") == 0)
          pd.outputString = pd.outputString + "STRINGLIST";
        else if(obj.objectType.compareTo("Packed Files") == 0)
          pd.outputString = pd.outputString + "PACKEDFILES";
        else
          pd.outputString = pd.outputString + obj.objectType.toUpperCase();

        pd.outputString = pd.outputString + " ";

        if(obj.dataType.compareTo("Integer") == 0)
          pd.outputString = pd.outputString + "I";
        else if(obj.dataType.compareTo("Float") == 0)
          pd.outputString = pd.outputString + "S";
        else if(obj.dataType.compareTo("Double") == 0)
          pd.outputString = pd.outputString + "D";
        else if(obj.dataType.compareTo("Char") == 0)
          pd.outputString = pd.outputString + "CHAR";
        else if(obj.dataType.compareTo("Byte") == 0)
          pd.outputString = pd.outputString + "B";
        else if(obj.dataType.compareTo("Simple Complex") == 0)
          pd.outputString = pd.outputString + "C";
        else if(obj.dataType.compareTo("Double Complex") == 0)
          pd.outputString = pd.outputString + "Z";

        pd.outputString = pd.outputString + " " + obj.name + "\n";
         
        pd.outputString = pd.outputString + obj.description;
        pd.outputString = pd.outputString + "\n";
      }
      pd.outputString = pd.outputString + "\n";
      pd.outputString = pd.outputString + "@CALLINGSEQUENCE\n";

      for(i=0; i < parent.problemDescription.argCount; i++){

        done = 0;
        firstTime = 1;
        tempString = "";
        for(j=0; j < parent.problemDescription.argCount; j++){

          if(parent.problemDescription.callSequence[j] == i+1){
            done = 1;
            if(firstTime == 1){
              tempString = tempString + "@ARG ";
              firstTime = 0;
            }
            else{
              tempString = tempString + ", ";
            }
            tempString = tempString + 
			parent.problemDescription.argmts.netsolveArgs[j];
          }
        }
        if(done == 1)
          pd.outputString = pd.outputString + tempString + "\n"; 
      }
      pd.outputString = pd.outputString + "\n";

      pd.outputString = pd.outputString + "@CODE\n";
      pd.outputString = pd.outputString + pd.code + "\n";
      pd.outputString = pd.outputString + "@END_CODE\n";
      
      return 1;
   }



  /**
   *
   * Check the problem name, problem path and problem description
   *
   */

   public String checkData()
   {
     if ((languageChoice.getSelectedItem()).compareTo("Languages") == 0)
       return new String("Select a language");
     if ((majorChoice.getSelectedItem()).compareTo("Major") == 0)
       return new String("Select a major");

     return new String("");
   }


  /**
     Does nothing
   **/

  public void itemStateChanged(ItemEvent e)
  {
   String source = (String)e.getItem();
    if(source.compareTo("About...") == 0) {
      parent.helpFrame.updateFrame("aboutHelp");
      parent.helpFrame.setVisible(true);
    }
    if(source.compareTo("Help on this page") == 0) {
      parent.helpFrame.updateFrame("miscHelp");
      parent.helpFrame.setVisible(true);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == saveButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == saveButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }

}

