/**
* SecondFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

public class ObjectFrame extends OkayFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public InOutFrame parent;
  public static boolean inApplet;

  public String type;
  public String objectName;

  private TextArea descriptionArea;
  private Choice dataTypeChoice;
  private Choice objectTypeChoice;

  GridBagLayout gridbag;
  GridBagConstraints c;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;

  /**
   * 
   * This is the constructor for the SecondFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public ObjectFrame(InOutFrame f, String Name, String IO) {
  
    super(IO + " Object " + Name);
    this.parent = f;
    this.inApplet = parent.inApplet;
    this.objectName = Name;
    this.type = IO;


    bottomPanel.add(errorMessage, "North");

    /* Take care of the top panel */
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    Label nameLabel = new Label(objectName);
    nameLabel.setForeground(Color.red);
    nameLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));

    Panel descriptionPanel = new Panel();
    descriptionPanel.setLayout(new BorderLayout(1,5));

    Label descriptionLabel = new Label("Object description:");
    descriptionLabel.setForeground(Color.red);
    descriptionLabel.setFont(new Font("Helvetica",Font.BOLD,LABEL_FONT_SIZE));
    descriptionArea = new TextArea("/** This is the default description **/\n",
                                10,40,TextArea.SCROLLBARS_VERTICAL_ONLY);
    descriptionArea.setEditable(true);

    descriptionPanel.add("North",descriptionLabel);
    descriptionPanel.add("South",descriptionArea);

    dataTypeChoice = new Choice();
    dataTypeChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    dataTypeChoice.setBackground(BUTTON_COLOR);
    dataTypeChoice.add("Data Type");
    dataTypeChoice.add("Integer");
    dataTypeChoice.add("Float");
    dataTypeChoice.add("Double");
    dataTypeChoice.add("Char");
    dataTypeChoice.add("Byte");
    dataTypeChoice.add("Simple Complex");
    dataTypeChoice.add("Double Complex");

    objectTypeChoice = new Choice();
    objectTypeChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    objectTypeChoice.setBackground(BUTTON_COLOR);
    objectTypeChoice.add("Object Type");
    objectTypeChoice.add("Scalar");
    objectTypeChoice.add("Vector");
    objectTypeChoice.add("Matrix");
    objectTypeChoice.add("Sparsematrix");
    objectTypeChoice.add("String");
    objectTypeChoice.add("File");
    objectTypeChoice.add("UPF");
    objectTypeChoice.add("String list");
    objectTypeChoice.add("Packed Files");

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(nameLabel, c);
    topPanel.add(nameLabel);

    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(descriptionPanel, c);
    topPanel.add(descriptionPanel);

    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(dataTypeChoice, c);
    topPanel.add(dataTypeChoice);

    c.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(objectTypeChoice, c);
    topPanel.add(objectTypeChoice);

    this.pack();
  }  

    public boolean in_applet() {
    return inApplet;
  }

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
    int i;
    Object source = event.getSource();

    if (source == okayButton) {
      // user clicked the "okay" button
       pushData();
       parent.updateStatusArea();
       this.setVisible(false);
    }
  }

  /**
   *
   * update the problem description
   *
   */

  public void pushData(){
  int i;


    if(type.compareTo("Input") == 0){
      
      for(i=0;i<parent.parent.problemDescription.inputObjects.size();i++){
        Objects obj = (Objects)(parent.parent.problemDescription.inputObjects.
						elementAt(i));
        if(obj.name.compareTo(objectName) == 0){
          obj.objectType = objectTypeChoice.getSelectedItem();
          if(obj.objectType.compareTo("String list") == 0){
            obj.dataType = "";
            obj.callObj = new StringList();
          }
          else if(obj.objectType.compareTo("Packed Files") == 0){
            obj.dataType = "";
            obj.callObj = new PackedFiles();
          }
          else if(obj.objectType.compareTo("File") == 0){
            obj.dataType = "";
            obj.callObj = new File();
          }
          else if(obj.objectType.compareTo("String") == 0){
            obj.dataType = "";
            obj.callObj = new Stringatt();
          }
          else if(obj.objectType.compareTo("UPF") == 0){
            obj.dataType = "";
            obj.callObj = new UPF();
          }
          else{
            obj.dataType = dataTypeChoice.getSelectedItem();
            if(obj.objectType.compareTo("Scalar") == 0){
              obj.callObj = new Scalar();
            }
            else if(obj.objectType.compareTo("Vector") == 0){
              obj.callObj = new Vector();
            }
            else if(obj.objectType.compareTo("Matrix") == 0){
              obj.callObj = new Matrix();
            }
            else if(obj.objectType.compareTo("Sparsematrix") == 0){
              obj.callObj = new SparseMatrix();
            }
          }

          if(obj.dataType.compareTo("Data Type") == 0)
            obj.dataType = "";
          if(obj.objectType.compareTo("Object Type") == 0)
            obj.objectType = "";

          obj.description = descriptionArea.getText();

          break;
        }
      }

   }
   else if(type.compareTo("Output") == 0){

      for(i=0;i<parent.parent.problemDescription.inputObjects.size();i++){
        Objects obj = (Objects)(parent.parent.problemDescription.outputObjects.
                                                elementAt(i));
        if(obj.name.compareTo(objectName) == 0){
          obj.objectType = objectTypeChoice.getSelectedItem();
          if(obj.objectType.compareTo("String list") == 0){
            obj.dataType = "";
            obj.callObj = new StringList();
          }
          else if(obj.objectType.compareTo("Packed Files") == 0){
            obj.dataType = "";
            obj.callObj = new PackedFiles();
          }
          else if(obj.objectType.compareTo("File") == 0){
            obj.dataType = "";
            obj.callObj = new File();
          }
          else if(obj.objectType.compareTo("String") == 0){
            obj.dataType = "";
            obj.callObj = new Stringatt();
          }
          else if(obj.objectType.compareTo("UPF") == 0){
            obj.dataType = "";
            obj.callObj = new UPF();
          }
          else{
            obj.dataType = dataTypeChoice.getSelectedItem();
            if(obj.objectType.compareTo("Scalar") == 0){
              obj.callObj = new Scalar();
            }
            else if(obj.objectType.compareTo("Vector") == 0){
              obj.callObj = new Vector();
            }
            else if(obj.objectType.compareTo("Matrix") == 0){
              obj.callObj = new Matrix();
            }
            else if(obj.objectType.compareTo("Sparsematrix") == 0){
              obj.callObj = new SparseMatrix();
            }
          }

          obj.description = descriptionArea.getText();

          break;
        }
      }

   }
  }

   public String checkData()
   {
     if ((dataTypeChoice.getSelectedItem()).compareTo("Data Type") == 0)
       return ("Missing data type");
     if ((objectTypeChoice.getSelectedItem()).compareTo("Object Type") == 0)
       return ("Missing object type");
     if (descriptionArea.getText().length() < 1)
       return new String("Missing object description");

     return new String("");
   }

  /**
     Does nothing
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      errorMessage.setText("");
    }
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
    Object source = e.getSource();

    if (source == okayButton)
    {
      String error = checkData();

      if (error.length() != 0)
      {
        errorMessage.setText(error);
      }
    }
  }



}

