/**
* SplitFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

abstract class OkayFrame extends Frame 
implements ActionListener, ItemListener, KeyListener, MouseListener
{
  Panel bottomPanel;

  protected Button okayButton;
  protected Panel topPanel;
  protected TextField errorMessage;

  static final int   BUTTON_FONT_SIZE 	= 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR 	= Color.lightGray;


  /**
   * 
   * This is the constructor for the SplitFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public OkayFrame(String title) {
  
    super(title);

    this.setLayout(new BorderLayout(2,2));

    /*
     * Constructing the bottom panel
     */

    okayButton = new Button("Okay");
    okayButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    okayButton.setBackground(BUTTON_COLOR);

    Panel buttonPanel = new Panel();
    buttonPanel.setBackground(BACKGROUND_COLOR);
    buttonPanel.add(okayButton);

    errorMessage = new TextField("");
    errorMessage.setEditable(false);
    errorMessage.setForeground(Color.red);

    topPanel = new Panel();
    topPanel.setBackground(BACKGROUND_COLOR);

    bottomPanel = new Panel();
    bottomPanel.setBackground(BACKGROUND_COLOR);
    bottomPanel.setLayout(new BorderLayout(2,2));

    bottomPanel.add("South",buttonPanel);

    this.setBackground(BACKGROUND_COLOR);
    this.add("North",topPanel);
    this.add("South",bottomPanel);

    okayButton.addActionListener(this);
    okayButton.addMouseListener(this);
  }  

}
