/**
* SplitFrame
*
* This class controls the graphical user interface for the Initial
* Screen.
* 
* @version 1.1, 26 Jan 1998 
* @author Henri Casanova (casanova@cs.utk.edu)
*
**/

import java.awt.*;
import java.awt.event.*;

abstract class SplitFrame extends Frame 
implements ActionListener, ItemListener, KeyListener, MouseListener
{
  protected Choice helpChoice;
  protected Button quitButton;
  protected Button prevButton;
  protected Button nextButton;
  protected Button saveButton;
  protected Panel topPanel;
  protected Panel navigatePanel;
  protected Panel quitPanel;
  protected Panel errorPanel;
  protected TextField errorMessage;

  GridBagLayout gridbag;
  GridBagConstraints c;

  static final int   BUTTON_FONT_SIZE 	= 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR 	= Color.lightGray;


  /**
   * 
   * This is the constructor for the SplitFrame class.  The purpose of
   * this method is to initialize and layout all the graphical components 
   * in the Initial Screen.  This is all done with standard AWT components.
   * 
   * @param f The parent frame.
   *
   **/

  public SplitFrame(String title) {
  
    super(title);

    this.setLayout(new BorderLayout(10,10));

    /*
     * Constructing the bottom panel
     */

    topPanel = new Panel();
    topPanel.setBackground(BACKGROUND_COLOR);
    gridbag = new GridBagLayout();
    c = new GridBagConstraints();
    topPanel.setLayout(gridbag);

    helpChoice = new Choice();
    helpChoice.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    helpChoice.setBackground(BUTTON_COLOR);
    helpChoice.add("Help");
    helpChoice.add("About...");
    helpChoice.add("Help on this page");

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.EAST;
    gridbag.setConstraints(helpChoice, c);
    topPanel.add(helpChoice);

    nextButton = new Button("Next>>");
    nextButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    nextButton.setBackground(BUTTON_COLOR);

    saveButton = new Button("Save");
    saveButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    saveButton.setBackground(BUTTON_COLOR);

    prevButton = new Button("<< Prev");
    prevButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    prevButton.setBackground(BUTTON_COLOR);

    quitButton = new Button("Quit");
    quitButton.setFont(new Font("Helvetica",Font.BOLD,BUTTON_FONT_SIZE));
    quitButton.setBackground(BUTTON_COLOR);

    Panel bottomPanel = new Panel();
    bottomPanel.setBackground(BACKGROUND_COLOR);
    bottomPanel.setLayout(new BorderLayout(2,2));

    Panel buttonPanel = new Panel();
    buttonPanel.setBackground(BACKGROUND_COLOR);
    buttonPanel.setLayout(new BorderLayout(1,1));

    quitPanel = new Panel();
    quitPanel.setBackground(BACKGROUND_COLOR);

    navigatePanel = new Panel();
    navigatePanel.setBackground(BACKGROUND_COLOR);

    buttonPanel.add("West",quitPanel);
    buttonPanel.add("East",navigatePanel);

    errorPanel = new Panel();
    errorPanel.setBackground(BACKGROUND_COLOR);

    errorMessage = new TextField("");
    errorMessage.setEditable(false);
    errorMessage.setForeground(Color.red);

    bottomPanel.add("North",errorPanel);
    bottomPanel.add("South",buttonPanel);

    this.setBackground(BACKGROUND_COLOR);
    this.add("North",topPanel);
    this.add("South",bottomPanel);

    helpChoice.addItemListener(this);
    quitButton.addActionListener(this);
    nextButton.addActionListener(this);
    saveButton.addActionListener(this);
    prevButton.addActionListener(this);

    nextButton.addMouseListener(this);
    saveButton.addMouseListener(this);
    prevButton.addMouseListener(this);
  }  

}
