/****
ToolBox
*
*
* This class controls the graphical user interface for the tollbox
* Screen.
*
*
****/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class ToolBox extends SplitFrame
 implements ActionListener, ItemListener, KeyListener, MouseListener {

  public PDGUI parent;
  public static boolean inApplet;

  Label descriptionLabel;
  Panel buttonsPanel;
  Button[] toolButtons;

  GridBagLayout buttonsGridbag;
  GridBagConstraints buttonsConstraints;

  int original_height;
  int original_width;

  private static final int LABEL_FONT_SIZE    = 14;
  static final int   BUTTON_FONT_SIZE   = 12;

  private static final Color BACKGROUND_COLOR   = Color.white;
  static final Color BUTTON_COLOR       = Color.lightGray;


  public ToolBox(PDGUI f) {
  
    super("Variable Toolbox");
    this.parent = f;
    this.inApplet = parent.inApplet;

    Panel descriptionPanel = new Panel();
    descriptionPanel.setLayout(new BorderLayout(1,1));

    descriptionLabel = new Label("The variables associated with the buttons are pointers or addresses");
    descriptionLabel.setForeground(Color.black);
    descriptionLabel.setFont(new Font("Helvetica",Font.PLAIN,
					LABEL_FONT_SIZE-5));

    descriptionPanel.add(descriptionLabel, "North");

    buttonsPanel = new Panel();
    buttonsGridbag = new GridBagLayout();
    buttonsConstraints = new GridBagConstraints();
    buttonsPanel.setLayout(buttonsGridbag);

    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(buttonsPanel, c);
    topPanel.add(buttonsPanel);
    c.fill = GridBagConstraints.NONE;
    c.anchor = GridBagConstraints.WEST;
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(descriptionLabel, c);
    topPanel.add(descriptionLabel);


    this.pack();

    Dimension d = this.getSize();
    original_height = d.height;
    original_width = d.width;

  }

  public void updateFrame(){
  int i, rowNumber;

    addButtons();

    buttonsPanel.removeAll();


    for(i=0; i<parent.problemDescription.argCount; i++){

      rowNumber = i/8;
      if(rowNumber == buttonsPanel.getComponentCount()){
      /* need to add a new Panel */
        Panel tempRowPanel = new Panel();
        tempRowPanel.add(toolButtons[i]);
        buttonsConstraints.gridwidth = GridBagConstraints.REMAINDER;
        buttonsConstraints.anchor = GridBagConstraints.WEST;
        buttonsGridbag.setConstraints(tempRowPanel, buttonsConstraints);
        buttonsPanel.add(tempRowPanel);
      }
      else{
      /* there is already a panel for this row */
        Panel tempRowPanel = (Panel)buttonsPanel.
					getComponent(rowNumber);
        tempRowPanel.add(toolButtons[i]);
      }
    }
    this.setSize(original_width + 40*(toolButtons.length),
        original_height+20*(buttonsPanel.getComponentCount()));
    this.validateTree();

  }

  public void addButtons(){
  int i, j, argIndex = 0;
  String variableString;


    toolButtons = new Button[parent.problemDescription.argCount];
    for(argIndex=0; argIndex < parent.problemDescription.argCount; argIndex++){

      variableString = parent.problemDescription.argmts.userArgs[argIndex];

      toolButtons[argIndex] = new Button(variableString);
      toolButtons[argIndex].setBackground(BUTTON_COLOR);
      toolButtons[argIndex].addActionListener(this);
    }
  }   

  /**
   *
   * If this is running as an applet, dispose of all
   * screens opened by the NetSolve Java interface.
   * Otherwise, only call the dispose method of the
   * parent class.
   *
   **/

  public synchronized void dispose() {
    super.dispose();
  }

  /**
   *
   * This is the method which is called whenever an action event happens.
   * First, determine the source of the event.  Then execute the code
   * associated with this event.
   *

   * @param event The event that was generated.
   *
   **/

  public void actionPerformed(ActionEvent event) {
  int i, pos;

    Object source = event.getSource();

    if (source == quitButton) {
      // user clicked the "quit" button
      if(inApplet)
        this.dispose();
      else
        System.exit(0);
    }
    else{
      Button tempButton = (Button)source;
      tempButton.setForeground(Color.white);
      String s = tempButton.getLabel();
      pos = parent.codeFrame.outputArea.getCaretPosition();
      parent.codeFrame.outputArea.insert(s, pos);
    }
  }

  /**
   *
   * Does nothing.
   *
   * @param e The event that was generated.
   *
   **/

  public void itemStateChanged(ItemEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyTyped(KeyEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyReleased(KeyEvent e)
  {
  }

  /**
   *
   * Mouse entering components
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseClicked(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing

   * @param e The key event that was generated.
   *
   **/

  public void mousePressed(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseReleased(MouseEvent e)
  {
  }


  /**
   *
   * Remove any error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseExited(MouseEvent e)
  {
  }

  /**
   *
   * May display an error message
   *
   * @param e The key event that was generated.
   *
   **/

  public void mouseEntered(MouseEvent e)
  {
  }

  /**
   *
   * Does nothing.
   *
   * @param e The key event that was generated.
   *
   **/

  public void keyPressed(KeyEvent e)

  {
  }

}
