/*****************************************************************/
/*      startproxy.c                                             */
/*      Henri Casanova, Dorian Arnold                            */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "client.h"
#include "proxy.h"
#include <unistd.h>
#include <signal.h>

int my_major;
int my_data_format;
int proxy_port;

/*
 * Start the proxy 
 *
 * This function starts the proxy as a separate process. It calls
 * waitForProxyToComeUp() which in turn establishes the life link.
 */
int startProxy(char *type,char *agent, int *proxy_pid, int *proxy_port)
{
  int pid;
  int port;
  NS_Socket_type sock;
  char *root;
  char *arch;

  root = getNetSolveRoot();
  if (root == NULL)
    return -1;
  arch = getNetSolveArch();
  if (arch == NULL)
  {
    free(root);
    return -1;
  }

  /* If there is already a running proxy, kill it !! */
  if (*proxy_pid)
    kill(*proxy_pid,SIGTERM);

#ifdef DEBUG
  fprintf (stderr, "startProxy(): Attempting to bind to port\n");
#endif
  /* Try to bind to a port */
  sock = bindToFirstAvailablePort(PORT_INIT,&port, 0);
  if (sock == -1)
  {
    fprintf(stderr, "Cannot bind to a port\n");
    ns_errno = NetSolveSystemError;
    free(root); free(arch);
    return -1;
  }

  /* Now one can fork */
  fflush(stdout); fflush(stderr);
  pid = fork();
  if (pid == -1)
  {
    perror("fork()");
    ns_errno = NetSolveCannotStartProxy;
    free(root); free(arch);
    return -1;
  }
  if (pid) /* Father */
  {
    close(sock);
    *proxy_pid = pid; 
    *proxy_port = port;
    free(arch); free(root);

#ifdef DEBUG
  fprintf (stderr, "startProxy(): Parent process calls waitForProxyToComeUp()\n");
#endif
    if (waitForProxyToComeUp() == -1)
    {
      perror("waitForProxyToComeUp()");
      ns_errno = NetSolveCannotStartProxy;
      return -1; 
    }
    ns_errno = NetSolveOK;

#ifdef DEBUG
  fprintf (stderr, "startProxy(): Parent process successfully returns\n");
#endif
    return 1;
  }
  else /* Child */
  {
    char path[256];
    int status;
    char **args;
    char buf[10];

    /* Start the proxy */
    
    if (!strcmp(type,"NetSolve"))
      sprintf(path,"%s/bin/%s/netsolveproxy",root,arch);
    else if (!strcmp(type,"Globus"))
      sprintf(path,"%s/bin/%s/globusproxy",root,arch);
    else {
      fprintf(stderr, "Unknown proxy type\n");
      exit(-1);
    }
    args = (char **)calloc(5,sizeof(char*));
    args[0] = strdup("ns_proxy");
    args[1] = strdup(agent);
    sprintf(buf,"%d",sock);
    args[2] = strdup(buf);
    sprintf(buf,"%d",port);
    args[3] = strdup(buf);
    args[4] = NULL;
    status = execv(path,args);
    fprintf(stderr, "Failed execv!\n");
    exit(-1);
  }
  return 1;
}
