#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#include "sparse_globals.h"
#include "itpack.h"
#include "./netsolve_itpack.h"
#include "./itpack_impl.h"
#include "itpack_auxs.h"
#include "matrix_auxs.h"
#include "parallel_auxs.h"
#include "driver/sparse.h"

int itpack_iterative_solve
(MPI_Comm comm, double *val,double *rhs,
 int first,int last, int nnzero,int *idx,int *ptr,
 double *sol, 
 double rtol,int maxit, int *conv,int *its
 );

int itpack_iterative_driver
(MPI_Comm comm, double *values,int *indices,int *pointers,
 int first,int local_size,
 double *rhs_vector,double *sol_vector,
 iterative_info_block info)
{
  itpack_info_block itpack_info = (itpack_info_block) info;
  int conv,its,nnz,ierr;

  ierr = crs_nnzeros(pointers,local_size,1,&nnz); ERR_RETURN(ierr);

  ierr = itpack_iterative_solve
    (comm,values,rhs_vector, first,first+local_size,nnz,
     indices,pointers,sol_vector,
     itpack_info->rtol,itpack_info->maxit,&conv,&its); ERR_RETURN(ierr);
  itpack_set_return_params(itpack_info,conv,its);

  return 0;
}

int itpack_iterative_solve
(MPI_Comm comm, double *val,double *rhs,
 int first,int last, int nnzero,int *idx,int *ptr,
 double *sol, 
 double rtol,int maxit, int *conv,int *its
 )
{
  int size=last-1,lwork,*iwork,iparm[12],info,flops;
  double *rwork,rparm[12];

  if (first!=1) ERR_REPORTi("Matrix distributed or strange; first=",first);

  /* parameters */
  dfault_(iparm,rparm);
  iparm[0] = maxit;   iparm[1]=0/*output*/; iparm[3] = 6/*ftn stdout*/;
  iparm[4] = 1/*nonsymm storage*/; iparm[8]=-2/*no redblack*/; iparm[11]=0;
  rparm[0]=rtol;

  /* work space */
  lwork = 8*size+4*maxit;
  ALLOCATE(iwork,3*size,int,"integer work space");
  ALLOCATE(rwork,lwork,double,"real work space");

  jcg_(&size,ptr,idx,val,rhs,sol,
	  iwork,&lwork,rwork,iparm,rparm,
	  &info,&flops);
  printf("iter: %d\n", iparm[0]);
  *its = iparm[0]; *conv = (*its<maxit);

  return 0;
}
  
