#include <stdlib.h>
#include "petsc.h"
#include "driver/sparse.h"
#include "petsc_auxs.h"
#include "./petsc_impl.h"

int petsc_params(direct_info_block *info,double rtol,int maxit,
		 double atol,double dtol)
{
  petsc_info_block petsc_info;

  petsc_info = (petsc_info_block) malloc
    (sizeof(struct petsc_info_block_));
  petsc_info->rtol = rtol; petsc_info->maxit = maxit;
  petsc_info->atol = atol; petsc_info->dtol = dtol;
  *info = (direct_info_block) petsc_info;

  return 0;
}

int petsc_set_return_params(iterative_info_block info,int conv,int its)
{
  petsc_info_block petsc_info = (petsc_info_block) info;

  petsc_info->conv = conv; petsc_info->its = its;

  return 0;
}

int petsc_get_return_params(iterative_info_block info,int *conv,int *its)
{
  petsc_info_block petsc_info = (petsc_info_block) info;

  *conv = petsc_info->conv; *its = petsc_info->its;

  return 0;
}

int petsc_matrix_from_crs
(MPI_Comm comm,int first,int local_size,
 Scalar *mv,int *idx,int *pointers, Mat *A)
{
  int last=first+local_size, row,col, ierr;

  {
    int *dband,*oband;
    dband = (int*) malloc((local_size+1)*sizeof(int));
    oband = (int*) malloc((local_size+1)*sizeof(int));
    for (row=0; row<local_size; row++) {
      dband[row]=0; oband[row]=0;
    }
    for (row=0; row<local_size; row++) {
      for (col=pointers[row]; col<pointers[row+1]; col++) {
	if (idx[col]>=first && idx[col]<last) {
	  dband[row]++;
	} else {
	  oband[row]++;
	}
      }
    }
    ierr = MatCreateMPIAIJ
      (comm,local_size,local_size,PETSC_DECIDE,PETSC_DECIDE,
       0,dband,0,oband,A); ERR_RETURN(ierr);
    free(dband); free(oband);
  }
  for (row=0; row<local_size; row++) {
    for (col=pointers[row]; col<pointers[row+1]; col++) {
      int Row=row+first;
      ierr = MatSetValues(*A,1,&Row,1,idx+col,mv+col,INSERT_VALUES);
      ERR_RETURN(ierr);
    }
  }
  ierr = MatAssemblyBegin(*A,MAT_FINAL_ASSEMBLY); ERR_RETURN(ierr);
  ierr = MatAssemblyEnd(*A,MAT_FINAL_ASSEMBLY); ERR_RETURN(ierr);

  return 0;
}

