/*****************************************************************/
/*      forkpetscservice.c                                       */
/*      Henri Casanova                                           */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "serverglobal.h"
#include "petscservice.h"
#include "serviceutil.h"
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

/*
 * forkPETSCService()
 */
void forkPETSCService(NS_Socket_type listening_socket, int ID,
                      int client_major, int lifelink_pid,
                      char *agent_name,
                      char *wd, char *serverhost,
                      int serverport, int restriction_index, 
                      char *xpath,char *xname,
                      NS_ProblemDesc *pd,NS_IPaddr_type proxy_IPaddr,
                      int proxy_port, char * localibp_name)
{
  int elapsed;
  time_t date1,date2;
  char command[412];
  char stdoutfile[256];
  char machinefile[256];
  FILE *f;
  char verdict[20];
  int rows_proc;
  NS_Socket_type sock;
  NS_Communicator *comm;
#ifdef MULTINODESEQ
  struct ibp_depot depot;

  depot.port = 6716;
  strcpy(depot.host, localibp_name);
#endif


  /***
   ***  FOR NOW, HARDCODED VALUES
   ***/
  rows_proc = 2;

  /* MPI */
  sprintf(machinefile,"%s/%s",global.netsolve_root_path,
                              global.MPInodefile);

  /* Creating the MPI virtual machine */
  sprintf(command,"%s/mpirun -machinefile %s -np %d %s/%s %s %d %d",
                        global.MPI_path,
                        machinefile, rows_proc,xpath,xname,
                        wd,client_major,rows_proc);
                        
  time(&date1);
  system(command);

  /* waiting for the done file */
  while((f = fopen("./done","r")) == NULL)
    sleep(3);
  time(&date2);

  /* Getting the verdict */
  fscanf(f,"%s",verdict);
  fclose(f);

  /* Notifying the proxy of completion */
  kill(lifelink_pid,SIGKILL);
  if (notifyProxy(proxy_IPaddr,proxy_port,NetSolveOK,ID,pd->job_id) == -1)
  {
    notifyServiceFinished(serverhost,serverport,restriction_index);
    cleanUp(wd);
    exit(0);
  }

  /* Listening on the socket */
  if ((sock = acceptConnection(listening_socket)) == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while accepting the connection.\n");
#endif
    notifyServiceFinished(serverhost,serverport,restriction_index);
    cleanUp(wd);
    exit(0);
  }
  comm = acceptTransaction(sock);
  if (comm == NULL)
  {
#ifdef VIEW
    fprintf(stderr,"Error while accepting the transaction.\n");
#endif
    notifyServiceFinished(serverhost,serverport,restriction_index);
    cleanUp(wd);
    exit(0);
  }

  /* Sending back the elpased time */
  elapsed = (int)(date2-date1);
  if (sendInt(comm,elapsed) == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while sending the elapsed time.\n");
#endif
    netsolvePerror(__FILE__,__LINE__);
    notifyServiceFinished(serverhost,serverport,restriction_index);
    endTransaction(comm);
    cleanUp(wd);
    exit(0);
  }

  /* sending back whatever stdout has been generated */
  sprintf(stdoutfile,"%s/netsolve-stdout",wd);
  if (sendFileAsString(comm,stdoutfile) == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while sending the stdout.\n");
#endif
    netsolvePerror(__FILE__,__LINE__);
    notifyServiceFinished(serverhost,serverport,restriction_index);
    endTransaction(comm);
    cleanUp(wd);
    exit(0);
  }

  /* Error ? */
  if (!strcmp(verdict,"ERROR"))
  {
#ifdef VIEW
    fprintf(stderr,"PETSC: ERROR\n");
#endif
    notifyServiceFinished(serverhost,serverport,restriction_index);
    sendInt(comm,NS_PROT_INTERNAL_FAILURE);
    endTransaction(comm);
    cleanUp(wd);
    exit(0);
  }

  /* SUCCESS !! */
  sendInt(comm,NS_PROT_SOLVED);

  /* Sending back the output */
#ifdef MULTINODESEQ
  if (sendOutputObjectsFromIBP(comm,&depot,pd) == -1)
#else
  if (sendOutputObjectsFromFiles(comm,pd) == -1)
#endif
  {
#ifdef VIEW
    fprintf(stderr,"Error while sending the output objects from files.\n");
#endif
    netsolvePerror(__FILE__,__LINE__);
    notifyServiceFinished(serverhost,serverport,restriction_index);
    endTransaction(comm);
    cleanUp(wd);
    exit(0);
  }

  /* Terminating the transaction */
  endTransaction(comm);
  notifyServiceFinished(serverhost,serverport,restriction_index);

#ifdef VIEW
  fprintf(stderr,"Completed\n");
#endif

  /* Cleaning up */
  cleanUp(wd);
  exit(0);
}
