/*****************************************************************/
/*      petscservice.c                                           */
/*      Henri Casanova                                           */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "serverglobal.h"
#include "petscservice.h"
#include "serviceutil.h"
#include <fcntl.h>
#include <string.h>

#ifdef PLAT_SUN4
int __main() { }
int MAIN_() { }
#endif



/*
 * PETSCservice()
 */
main(int argc, char **argv)
{
  int mypnum;
  int rtn;
  int client_major;
  char *wd;
  int i,j;
  int solved;
  int fd;
  NS_ProblemDesc *pd;
  char stdoutfile[256];
  int stdoutfd;
  char buffer[256];


  /* Initialize the MPI command line arguments */
  MPI_Init(&argc,&argv);

  /* Who am I ? */
  if (MPI_Comm_rank(MPI_COMM_WORLD,&MYROW) != MPI_SUCCESS)
  {
    createDoneFile("ERROR");
    exit(0);
  }

  /* Parse the command line */

  if (argc != 4)
  {
#ifdef VIEW
    fprintf(stderr,"PETSCService: wrong number of command line args.\n");
#endif
    createDoneFile("ERROR");
    exit(0);
  }

  wd = argv[1];
  client_major = atoi(argv[2]);
  NUM_PROCS = atoi(argv[3]);

  chdir(wd);

#ifdef VIEW
  if ((MYROW==0))
    fprintf(stderr,"MPI Grid initialized: %d processors\n",NUM_PROCS);
#endif

  /* Get the Problem Descriptor */
  sprintf(buffer,"./pb_desc");
  fd = open(buffer,O_RDONLY,0666);
  if (fd < 0)
  {
#ifdef VIEW
    fprintf(stderr,"Impossible to open the Problem Descriptor file.\n");
#endif
    petsc_exit("ERROR");
  }
  pd = readProblemDescFromFile(fd);
  close(fd);
  
  if (pd == NULL)
  {
#ifdef VIEW
    fprintf(stderr,"Error while reading the Problem Descriptor file.\n");
#endif
    petsc_exit("ERROR");
  }

#ifdef VIEW
  if (MYROW==0)
    fprintf(stderr,"Solving '%s'\n",pd->nickname);
#endif

  if (PETSCReadInputObjectsFromFiles(pd) == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while reading the input objects.\n");
#endif
    petsc_exit("ERROR");
  }

  /* Initializes the output objects */
  if (initOutputObjects(pd) == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while initializing the output objects.\n");
#endif
    netsolvePerror(__FILE__,__LINE__);
    createDoneFile("ERROR");
    exit(0);
  }

  /* Redirecting the stdout */
  sprintf(stdoutfile,"%s/netsolve-stdout",wd);
  stdoutfd = redirectStdout(stdoutfile);
  if (stdoutfd == -1)
  {
#ifdef VIEW
    fprintf(stderr,"Error while redirecting stdout\n");
#endif
    createDoneFile("ERROR");
    exit(0);
  } 

  /* Solve the problem */
  solved = solve(pd,pd->input_objects,pd->output_objects);

  fflush(stdout);
  close(stdoutfd);

  if (solved == NS_PROT_SOLVED)
  {
    if (PETSCWriteOutputObjectsToFiles(client_major,pd) == -1)
    {
#ifdef VIEW
      fprintf(stderr,"Error while writing the output objects.\n");
#endif
      petsc_exit("ERROR");
    }
    petsc_exit("DONE");
  }
  else
    petsc_exit("ERROR");

}

/*
 * initOutputObjects()
 */
int initOutputObjects(NS_ProblemDesc *pd)
{
  NS_Object **list = pd->output_objects;
  int nb = pd->nb_output_objects;
  char buffer[256];
  int i;

  for (i=0;i<nb;i++)
  {
    /* do special initializations */
    switch(list[i]->object_type)
    {
      case NETSOLVE_FILE: /* Create a file name */
        sprintf(buffer,"fileoutput%d",i);
        list[i]->attributes.file_attributes.filename = strdup(buffer);
        break;
      case NETSOLVE_MATRIX: /* Set the major */
        list[i]->attributes.matrix_attributes.major = pd->major;
        list[i]->attributes.matrix_attributes.l = -1;
        break;
      default: /* Do nothing */
        break;
    }
  }
  return 1;
}
