/*****************************************************************/
/*      generateservice.c                                        */
/*      Henri Casanova, Dorian Arnold                            */
/*---------------------------------------------------------------*/
/*****************************************************************/

#include "core.h"
#include "serverglobal.h"
#include "generateservice.h"
#include "standardservice.h"
#include "condorservice.h"
#include "petscservice.h"
#include "scalapackservice.h"
#include "netsolveupf.h"
#include "netsolveftp.h"
#include "serviceutil.h"
#include "lifelink.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

/*
 * generateServiceProcess
 *
 */
void generateServiceProcess(NS_Communicator *comm, int restriction_index,
                            NS_ProblemDesc *pd,char *agent_name,
                            NS_IPaddr_type proxy_IPaddr, int proxy_port, char *localibp_name)
{
  int ID;
  int fd;
  int upf,customized,ftp;
  char tmpdir[256];
  int tag;
  char *xname;
  char *xpath;
  char problemdescfile[256];
  char buffer[256];
  int status;
  char *arch;
  int count=0;
  NS_Socket_type listening_socket = 0;
  NS_Socket_type sock;
  int my_port;
  NS_Communicator *comm_to_client;
  int client_major;
  int my_pid;
  int lifelink_pid;
#ifdef MULTINODESEQ
  struct ibp_depot depot;
#endif
  int multinode_spawn = proxy_IPaddr == 0;

  if(!multinode_spawn) {
    if (fork())
      return; 

    close(global.sock);
  }

  fprintf(stderr, "In generateservice: multinode_spawn = %i\n", multinode_spawn);


  /* Setup local IBP */
#ifdef MULTINODESEQ
  depot.port = 6716;
  strcpy(depot.host,localibp_name);
#endif
  
  if(!multinode_spawn) {
    /* Open a new socket on a port */
    listening_socket = bindToFirstAvailablePort(3456,&my_port,0);
    if (listening_socket == -1)
      {
	fprintf(stderr,"Impossible to bind to a port\n");
	if (sendInt(comm,-1) == -1)
	  netsolvePerror(__FILE__,__LINE__);
	endTransaction(comm);
	notifyServiceFinished(global.my_self->host_desc->hostname,
			      global.my_self->port,restriction_index);
	exit(0);
      }


    /* listening on the socket */
    listen(listening_socket,MAX_CONNECTIONS);
  }

  /*********************************************/
  /* Place myself in a new temporary directory */
  /*********************************************/
  while(1)
  {
/*
    if (!strcmp(pd->customized,"ITER_SOLVE") ||
                !strcmp(pd->customized,"SCALAPACK")){
*/
    if (!strcmp(pd->customized,"ITER_SOLVE") || 
			!strcmp(pd->customized,"SCALAPACK")){
      sprintf(tmpdir,"%s/src/Server/netsolve-tmp-%d",
                global.netsolve_root_path, (int)getpid()+count);
    }
    else{
      sprintf(tmpdir,"%s/netsolve-tmp-%d",global.scratch_path,
                        (int)getpid()+count);
    }
    if (!access(tmpdir,F_OK))
      count++;
    else
      break;
  }
  if (mkdir(tmpdir,0777))
  {
    fprintf(stderr,"Impossible to create directory %s\n",tmpdir);
    if(!multinode_spawn) {
      if (sendInt(comm,-1) == -1)
	netsolvePerror(__FILE__,__LINE__);
      notifyServiceFinished(global.my_self->host_desc->hostname,
			    global.my_self->port,restriction_index);
    }
    exit(0);    
  } 

  if (chdir(tmpdir) == -1)
  {
    fprintf(stderr,"Impossible to chdir to %s\n",tmpdir);
    if(!multinode_spawn) {
      if (sendInt(comm,-1) == -1)
	netsolvePerror(__FILE__,__LINE__);
      endTransaction(comm);
      notifyServiceFinished(global.my_self->host_desc->hostname,
			    global.my_self->port,restriction_index);
    }
    cleanUp(tmpdir);
    exit(0); 
  }


  /*****************************************/
  /* Save the problem descriptor to a file */
  /*****************************************/

  if(multinode_spawn) {
    sprintf(problemdescfile, "./pb_desc_%i", pd->job_id);
    fprintf(stderr, "Writing pb_desc to \"%s\"\n", problemdescfile);
  } else {
    sprintf(problemdescfile,"./pb_desc");
  }
  fd = open(problemdescfile,O_WRONLY|O_CREAT,0666);
  if (fd < 0)
  {
    fprintf(stderr,"Impossible to create file '%s'\n",problemdescfile);
    if(!multinode_spawn) {
      if (sendInt(comm,-1) == -1)
	netsolvePerror(__FILE__,__LINE__);
      endTransaction(comm);
      notifyServiceFinished(global.my_self->host_desc->hostname,
			    global.my_self->port,restriction_index);
    }
    cleanUp(tmpdir);
    exit(0);
  }

  fprintf(stderr, "Will use file \"%s\" for problem description WRITE...\n",
	  problemdescfile);

  if(pd->input_names) {
    fprintf(stderr, "We HAVE input names\n");
  } else {
    fprintf(stderr, "WE DON'T HAVE INPUT NAMES!!\n");
  }

  if (writeProblemDescToFile(fd,pd) == -1)
  {
    fprintf(stderr,"Impossible to save problem descriptor to file %s\n",
           problemdescfile);
    if(!multinode_spawn) {
      if (sendInt(comm,-1) == -1)
	netsolvePerror(__FILE__,__LINE__);
    } else {
      sendInt(comm, NS_PROT_NOT_ALLOWED);
    }
    endTransaction(comm);
    if(!multinode_spawn) {
      notifyServiceFinished(global.my_self->host_desc->hostname,
			    global.my_self->port,restriction_index);
    }
    cleanUp(tmpdir);
    exit(0);
  }
  close(fd);

  /*******************************/
  /* Determine problem specifics */
  /*******************************/
  upf = containsUPF(pd);
  customized = strcmp(pd->customized,"NONE");
  ftp = pd->ftp;

  /***************************************************/
  /* Set the default path and name of the executable */
  /***************************************************/
  arch = getNetSolveArch();
  sprintf(buffer,"%s/bin/%s/",
            global.netsolve_root_path,arch);
  free(arch);
  xpath = strdup(buffer);
  sprintf(buffer,"service-%s",pd->file);
  xname = strdup(buffer);
  
  /**************************************************/
  /* Send back my new port & pid                    */
  /**************************************************/
  if(!multinode_spawn) {
    my_pid = getpid();
    if ((sendInt(comm,my_port) == -1) ||
	(sendInt(comm,my_pid) == -1))
      {
	endTransaction(comm);
	notifyServiceFinished(global.my_self->host_desc->hostname,
			      global.my_self->port,restriction_index);
	cleanUp(tmpdir);
	exit(0);
      } 
  }

  if(!multinode_spawn) {
    /* cut the connection to the client proxy */
    endTransaction(comm);
    comm = 0;
  }

  /*************************************************************/
  /* Use the standard service                                  */
  /* that will read the objects in input and send back the ack */
  /*************************************************************/

  if ((!ftp) && (!upf) && (!customized))
  {
    fprintf(stderr, "Okeydokey - about to exec the service!\n");

    if(multinode_spawn) {
      forkStandardService(listening_socket, -1, -1, -1, agent_name,
			  tmpdir,
			  global.my_self->host_desc->hostname,
			  global.my_self->port,
			  restriction_index,
			  xpath, xname, pd, proxy_IPaddr, proxy_port, NOFILES,   
			  pd->job_id,
			  localibp_name);
    } else {
      forkStandardNoFileService(listening_socket,
				agent_name,
				tmpdir,
				global.my_self->host_desc->hostname,
				global.my_self->port,
				restriction_index,
				xpath,
				xname,
				pd,
				proxy_IPaddr,
				proxy_port, 
				localibp_name);
    }
    cleanUp(tmpdir);
    exit(0);
  }

  if(multinode_spawn) {
    fprintf(stderr, "No way ! we're not running file services as multinode spawns!\n");
    exit(0);
  }

  /*********************************/
  /* Read all the objects to files */
  /*********************************/

  /* Wait for a connection from the client */
  sock = acceptConnection(listening_socket);
  if (sock == -1)
  {
    fprintf(stderr,"Error while accepting client connection\n");
    notifyServiceFinished(global.my_self->host_desc->hostname,
                          global.my_self->port,restriction_index);
    cleanUp(tmpdir);
    exit(0);
  }
#ifdef DEBUG
  fprintf(stderr,"Got connection for input objects\n");
#endif

  comm_to_client = acceptTransaction(sock);
  if (comm_to_client == NULL)
  {
    fprintf(stderr,"Error while accepting client transaction\n");
    notifyServiceFinished(global.my_self->host_desc->hostname,
                          global.my_self->port,restriction_index);
    cleanUp(tmpdir);
    exit(0);
  }

#ifdef KERBEROS5
  if (global.require_auth) {
    /* tell client we need credentials */
    tag = NS_PROT_KRB5_AUTH_REQUIRED;
    if (sendInt (comm_to_client, tag) == -1)
    {
      netsolvePerror (__FILE__,__LINE__);
      freeProblemDesc (pd);
      return;
    }

    /* get credentials from client */
    if (RecvKerberos5Credentials (comm_to_client) < 0) {
      /*
       * if we didn't like the credentials; tell client that the
       * authentication failed, and this will be the error code
       * associated with the problem request.  otherwise, fall
       * through to other error checking below
       */
      tag = NS_PROT_AUTH_FAILED;
      if (sendInt (comm_to_client, tag) == -1)
      {
        netsolvePerror (__FILE__,__LINE__);
        freeProblemDesc (pd);
        return;
      }
      return;
    }
    tag = NS_PROT_ACCEPTED;
    if (sendInt (comm_to_client, tag) == -1)
    {
      netsolvePerror (__FILE__,__LINE__);
      freeProblemDesc (pd);
      return;
    }
  }
  else{
    tag = NS_PROT_ACCEPTED;
    if (sendInt (comm_to_client, tag) == -1)
    {
      netsolvePerror (__FILE__,__LINE__);
      freeProblemDesc (pd);
      return;
    }
  }
#else
  tag = NS_PROT_ACCEPTED;
  if (sendInt (comm_to_client, tag) == -1)
  {
    netsolvePerror (__FILE__,__LINE__);
    freeProblemDesc (pd);
    return;
  }
#endif

  /* receiving the ID and the major */
  if ((recvInt(comm_to_client,&ID) == -1) ||
      (recvInt(comm_to_client,&client_major) == -1))
  {
    fprintf(stderr,"Error while accepting client transaction\n");
    /* Trying to send something back anyway !! */
    if (sendInt(comm_to_client,NS_PROT_INTERNAL_FAILURE) == -1)
      netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_client);
    notifyServiceFinished(global.my_self->host_desc->hostname,
                          global.my_self->port,restriction_index);
    cleanUp(tmpdir);
    exit(0);
  }

  /* Receiving the input objects */
#ifdef MULTINODESEQ
  if (recvInputObjectsToIBP(comm_to_client,&depot,pd) == -1)
#else
  if (recvInputObjectsToFiles(comm_to_client,pd) == -1)
#endif
  {
    fprintf(stderr,"Error while receiving input objects to files\n");
    /* Trying to send something back anyway !! */
    if (sendInt(comm_to_client,NS_PROT_INTERNAL_FAILURE) == -1)
      netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_client);
    notifyServiceFinished(global.my_self->host_desc->hostname,
                          global.my_self->port,restriction_index);
    cleanUp(tmpdir);
    exit(0);
  }

  /* Cutting the connection to the client */
  endTransaction(comm_to_client);

  /*************************************************************/
  /* Start the life link to the proxy                          */
  /*************************************************************/
  if ((lifelink_pid = startLifeLink(proxy_IPaddr,proxy_port,NS_PROT_JOB_REPORT,
            ID,NetSolveServerError)) == -1)
  {
    fprintf(stderr,"Warning: %s,%d: cannot start life link....\n",
           __FILE__,__LINE__);
  }

  fprintf(stderr, "*started lifelink pd->job_id is still %i\n", pd->job_id);

  /* Process possible UPFs */
  if (upf)
  {
#ifdef MULTINODESEQ
    fprintf(stderr, "Oh, horror !  UPF not supported\n");
    exit(0);
#else
    status = processUPF(pd,tmpdir,&xname,&xpath);
    if (status  == -1)
    {
      kill(lifelink_pid,SIGKILL);
      notifyProxy(proxy_IPaddr,proxy_port,ns_errno,ID,pd->job_id);
      notifyServiceFinished(global.my_self->host_desc->hostname,
                            global.my_self->port,restriction_index);
      cleanUp(tmpdir);
      exit(0);
    }
#endif
  }

  if (ftp)
  {
    status = processFTP(pd,tmpdir,&xname,&xpath);
    if (status == -1)
    {
      kill(lifelink_pid,SIGKILL);
      notifyProxy(proxy_IPaddr,proxy_port,ns_errno,ID,pd->job_id);
      notifyServiceFinished(global.my_self->host_desc->hostname,
                            global.my_self->port,restriction_index);
      cleanUp(tmpdir);
      exit(0);

    }
  }
 
  if (!customized)
  {
    /* that will send back the ack */
    forkStandardFileService(listening_socket,ID,client_major,
                            lifelink_pid,agent_name,tmpdir,
                            global.my_self->host_desc->hostname,
                            global.my_self->port,restriction_index,
                            xpath,xname,pd,proxy_IPaddr,proxy_port);
  }

  if (!strcmp(pd->customized,"SCALAPACK"))
  {
    forkScaLAPACKService(listening_socket,ID,client_major,
                         lifelink_pid,agent_name,tmpdir,
                         global.my_self->host_desc->hostname,
                         global.my_self->port,restriction_index,
                         xpath,xname,pd,proxy_IPaddr,proxy_port,localibp_name);
  }
  else if (!strcmp(pd->customized,"CONDOR"))
  {
    forkCondorService(listening_socket,ID,client_major,
                      lifelink_pid,agent_name,tmpdir,
                      global.my_self->host_desc->hostname,
                      global.my_self->port,restriction_index,
                      xpath,xname,pd,proxy_IPaddr,proxy_port,localibp_name);
  }
  else if (!strcmp(pd->customized,"PETSC"))
  {
    forkPETSCService(listening_socket,ID,client_major,
                     lifelink_pid,agent_name,tmpdir,
                     global.my_self->host_desc->hostname,
                     global.my_self->port,restriction_index,
                     xpath,xname,pd,proxy_IPaddr,proxy_port,localibp_name);
  }
  else if (!strcmp(pd->customized,"ITER_SOLVE"))
  {
    forkiterativeSolverService(listening_socket,ID,client_major,
                     lifelink_pid,agent_name,tmpdir,
                     global.my_self->host_desc->hostname,
                     global.my_self->port,restriction_index,
                     xpath,xname,pd,proxy_IPaddr,proxy_port,localibp_name);
  }
  else
  {
#ifdef VIEW
    fprintf(stderr,"Unknown Customization\n");
#endif
    kill(lifelink_pid,SIGKILL);
    notifyProxy(proxy_IPaddr,proxy_port,NetSolveInternalError,ID,pd->job_id);
    notifyServiceFinished(global.my_self->host_desc->hostname,
                          global.my_self->port,restriction_index);
    cleanUp(tmpdir);
    exit(0);

  }

  exit(0);
}
