/*****************************************************************/
/*      server_process_message_sequence.c                        */
/*      Dieter Bachmann                                          */
/*---------------------------------------------------------------*/
/*                                                               */
/*****************************************************************/

#include "core.h"
#include "serverglobal.h"
#include "serverprocessmessage.h"
#include "workloadmanager.h"
#include "pong.h"
#include "generateservice.h"
#include "abort.h"
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

#define VIEW 1

/*
 * processProblemSolve_sequence()
 */
void processProblemSolve_sequence(NS_Communicator *comm)
{
  NS_Node     *top_node;
  NS_Node     *tmp_node;
  NS_ProblemDesc **plist;
  int restriction_index;
  NS_IPaddr_type proxy_IPaddr;
  int proxy_port;
  char *agent;
  char * ibp_server_name;

  top_node = netsl_recv_Node_with_Servers(comm);
  if (top_node == NULL)
  {
    netsolvePerror(__FILE__,__LINE__);
    return;
  }
  if ((recvString(comm,&agent) == -1) ||
      (recvIPaddr(comm,&proxy_IPaddr) == -1) ||
      (recvInt(comm,&proxy_port) == -1))
  {
    netsolvePerror(__FILE__,__LINE__);
    return;
  }
  
  tmp_node = top_node;
  while(tmp_node != NULL){  /* for all nodes */

    fprintf(stderr, "Received sequence node with "
	    "ID=%i and job_id=%i\n", tmp_node->id,
	    tmp_node->pd->job_id);

      plist = lookupProblem(tmp_node->pd->nickname);
      if (*plist == NULL){
	  if (sendInt(comm,NS_PROT_PROBLEM_NOT_FOUND) == -1){
	      netsolvePerror(__FILE__,__LINE__);
	      netsl_delete_Nodes(top_node);
	      return;
	  }
	  free(plist);
	  fprintf(stderr, "Told someone that the problem (%s) could not be found\n",
		  tmp_node->pd->nickname);
	  return;
      }
      if (!compareProblemDesc(tmp_node->pd, *plist)){
	  if (sendInt(comm,NS_PROT_BAD_SPECIFICATION) == -1){
	      netsolvePerror(__FILE__,__LINE__);
	      netsl_delete_Nodes(top_node);
	      return;
	  }
	  free(plist);
	  netsl_delete_Nodes(top_node);
	  fprintf(stderr, "Told someone that the speification was bad\n");
	  return;
      }
      /* here we used to copy *plist to pd presumably
       * because of code duplication from 
       * server_process_message.c
       */
      free(plist);
      tmp_node=tmp_node->next; /* next node */
  } /* for all nodes */

  /* Workload Threshhold */
  if (global.my_self->workload_threshhold != -1)
  {
    if (global.my_self->workload_threshhold < getMyWorkload())
    {
#ifdef VIEW
      fprintf(stderr,"Job refused: Workload beyond threshhold\n");
#endif
      if (sendInt(comm,NS_PROT_NOT_ALLOWED) == -1)
      {
        netsolvePerror(__FILE__,__LINE__);
        return;
      }
      return;
    }
  }

  /* Restrictions */
  restriction_index = checkRestriction(comm,global.my_self->restrictions,
                  global.my_self->nb_restrictions);
  if (restriction_index == -1)
  {
    if (ns_errno == NetSolveNotAllowed)
    {
#ifdef VIEW
      fprintf(stderr,"Job refused: Not an allowed client at this time\n");
#endif
      if (sendInt(comm,NS_PROT_NOT_ALLOWED) == -1)
      {
        netsolvePerror(__FILE__,__LINE__);
        return;
      }
      return;
    }
    if (ns_errno == NetSolveSystemError)
    {
        /* nothing */
    }
  }

  /* At this point, either we had a system Error or
     we found the restirction slot */

  if (sendInt(comm,NS_PROT_ACCEPTED) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    return;
  }

  ibp_server_name = reverse_lookup(top_node->servers->IBPaddrlist);
  if(!ibp_server_name) ibp_server_name = "";

  /*  forking off the scheduleer for the actual sequence */
  generateServiceProcess_sequence(comm,restriction_index,
				  top_node,agent,
				  proxy_IPaddr,proxy_port,ibp_server_name);

  netsl_delete_Nodes(top_node); /* free node memory (after fork)*/
  return;
}

