#include "core.h"
#include "mnemonics.h"
#include "client.h"

int get_agent_list(NS_AgentDesc ***list, char *agent)
{
  int nb_agents,i;
  NS_Communicator *comm_to_agent;
 
  /* contact the agent */
  comm_to_agent = contactHost(agent,0,AGENT_PORT,DATA_XDR);
  if (comm_to_agent == NULL)
  {
    fprintf(stdout, "Cannot contact agent on %s\n", agent); 
    return -1;
  }
 
  /* Sending */
  if (sendInt(comm_to_agent,NS_PROT_AGENT_LIST) == -1)
  {
    fprintf(stdout, "Network Error\n");
    endTransaction(comm_to_agent);
    return -1;
  }
 
  if (recvInt(comm_to_agent,&nb_agents) == -1)
  {
    fprintf(stdout, "Network Error\n");
    endTransaction(comm_to_agent);
    return -1;
  }
 
  *list = (NS_AgentDesc **)calloc(nb_agents,sizeof(NS_AgentDesc*));
  for (i=0;i<nb_agents;i++)
  {
    (*list)[i] = recvAgentDesc(comm_to_agent);
    if (*(list)[i] == NULL)
    {
      fprintf(stdout, "Network Error\n");
      endTransaction(comm_to_agent);
      return -1;
    }
  }

  endTransaction(comm_to_agent);
  return nb_agents;
}

int get_server_list(NS_ServerDesc ***list, char *agent)
{
  int nb_servers,i;
  NS_Communicator *comm_to_agent;
 
  /* contact the agent */
  comm_to_agent = contactHost(agent,0,AGENT_PORT,DATA_XDR);
  if (comm_to_agent == NULL)
  {
    fprintf(stdout, "Cannot contact agent on %s\n", agent);
    return -1;
  }
 
  /* Sending */
  if (sendInt(comm_to_agent,NS_PROT_SERVER_LIST) == -1)
  {
    fprintf(stdout, "Network Error\n");
    endTransaction(comm_to_agent);
    return -1;
  }
 
  if (recvInt(comm_to_agent,&nb_servers) == -1)
  {
    fprintf(stdout, "Network Error\n");
    endTransaction(comm_to_agent);
    return -1;
  }
 
  (*list) = (NS_ServerDesc **)calloc(nb_servers,sizeof(NS_AgentDesc*));
  for (i=0;i<nb_servers;i++)
  {
    (*list)[i] = recvServerDesc(comm_to_agent);
    if (*(list)[i] == NULL)
    {
      fprintf(stdout, "Network Error\n");
      endTransaction(comm_to_agent);
      return -1;
    }
  }
 
  endTransaction(comm_to_agent);
  return nb_servers;
}

int get_problem_list(NS_ProblemDesc ***list, char * agent)
{
  int nb_problems,i;
  NS_Communicator *comm_to_agent;

  /* contact the agent */
  comm_to_agent = contactHost(agent,0,AGENT_PORT,DATA_XDR);
  if (comm_to_agent == NULL)
  {
    netsolvePerror(__FILE__,__LINE__);
    return -1;
  }
  
  /* Sending */
  if (sendInt(comm_to_agent,NS_PROT_PROBLEM_LIST) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_agent);
    return -1;
  }

  if (recvInt(comm_to_agent,&nb_problems) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_agent);
    return -1;
  }

  *list = (NS_ProblemDesc **)calloc(nb_problems,sizeof(NS_ProblemDesc*));
  for (i=0;i<nb_problems;i++)
  {
    (*list)[i] = recvProblemDesc(comm_to_agent);
    if ((*list)[i] == NULL)
    {
      netsolvePerror(__FILE__,__LINE__);
      endTransaction(comm_to_agent);
      return -1;
    }
  }
  endTransaction(comm_to_agent);

  return nb_problems;
}

int get_problem(char * nickname, NS_ProblemDesc **pd, char *agent)
{
  NS_Communicator *comm_to_agent;
  int tag;

  /* contact the agent */
  comm_to_agent = contactHost(agent,0,AGENT_PORT,DATA_XDR);
  if (comm_to_agent == NULL)
  {
    netsolvePerror(__FILE__,__LINE__);
    return -1;
  }

  /* Sending */
  if (sendInt(comm_to_agent,NS_PROT_PROBLEM_INFO) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_agent);
    return -1;
  }
  if (sendString(comm_to_agent,nickname) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_agent);
    return -1;
  }

  /* Receiving */
  if (recvInt(comm_to_agent,&tag) == -1)
  {
    netsolvePerror(__FILE__,__LINE__);
    endTransaction(comm_to_agent);
    return -1;
  }

  /* Forward the tag */
  if (tag != NS_PROT_PROBLEM_DESC)
  {
    return -1;
  }
  else
  {
    *pd = recvProblemDesc(comm_to_agent);
    if (*pd == NULL)
    {
      netsolvePerror(__FILE__,__LINE__);
	  endTransaction(comm_to_agent);
	  return -1;
    }
  }
  endTransaction(comm_to_agent);

  return 0;
}
